import os
from dotenv import load_dotenv
from urllib.parse import urlencode, urljoin

from ai_datahive.collectors import BaseCollector
from ai_datahive.collectors.utils.scraping_helper import get_request_as_json
from ai_datahive.utils.datetime_helper import to_periodic_format

from ai_datahive.collectors.models import Media


class CivitaiCollector(BaseCollector):
    VALID_PERIODS = ['AllTime', 'Year', 'Month', 'Week', 'Day']
    VALID_MEDIA_TYPES = ['image', 'video', 'model']
    VALID_NSFW_LEVELS = [None, 'Soft', 'Mature', 'X']
    VALID_SORTS = ['Most Reactions', 'Most Buzz', 'Most Comments', 'Most Collected', 'Oldest']

    def __init__(self, creator_name='CivitaiCollector', media_type='image', period='Day', sort='Most Reactions',
                 nsfw=None, tags=None, limit=1):
        self.creator_name = creator_name
        if tags is None:
            self.tags = self.build_tags(media_type, period, sort, nsfw)
        else:
            self.tags = tags
        self.limit = limit

        self.validate_parameters(media_type, period, nsfw, sort)

        # TODO Right now, there is no way to determine videos directly from Civitai API
        # START - Workaround as long as there is no way to get videos directly from Civitai
        if media_type == 'video':
            self.media_type = 'image'
        else:
            self.media_type = media_type
        # END - Workaround as long as there is no way to get videos directly from Civitai

        base_url = os.getenv('CIVITAI_API_URL', '')
        self.civitai_url = self.build_url(base_url, media_type, sort, nsfw, period, limit)

        super().__init__(creator_name=self.creator_name, content_type=Media)

    def build_tags(self, media_type, period, sort, nsfw):
        tags = 'civitai, '
        tags += media_type.lower() + ', '
        tags += to_periodic_format(period) + ', '
        tags += sort.lower() + ', '

        if nsfw:
            tags += nsfw.lower() + ', '

        tags = tags.rstrip(', ')
        return tags

    def validate_parameters(self, media_type, period, nsfw, sort):
        if period not in self.VALID_PERIODS:
            raise ValueError(f"period must be one of {self.VALID_PERIODS}")
        if media_type not in self.VALID_MEDIA_TYPES:
            raise ValueError(f"media_type must be one of {self.VALID_MEDIA_TYPES}")
        if nsfw not in self.VALID_NSFW_LEVELS:
            raise ValueError(f"nsfw must be one of {self.VALID_NSFW_LEVELS}")
        if sort not in self.VALID_SORTS:
            raise ValueError(f"sort must be one of {self.VALID_SORTS}")

    def build_url(self, base_url, media_type, sort, nsfw, period, limit):
        path = f"{media_type}s"  # Append 's' to make it plural (image -> images, video -> videos, model -> models)
        params = {
            'nsfw': nsfw if nsfw is not None else 'None',
            'limit': limit,
            'period': period,
            'sort': sort
        }
        query_string = urlencode({k: v for k, v in params.items() if v is not None})  # Exclude None values
        return urljoin(base_url, path) + '?' + query_string

    def retrieve(self):
        data = get_request_as_json(self.civitai_url)
        if isinstance(data, dict):
            media = self.convert_to_media(data)
            return media
        else:
            # Fehlerbehandlung
            print(f"Es gab einen Fehler bei der Anfrage: {data}")

    def convert_to_media(self, data):
        result = []
        if data:
            for item in data['items']:
                media = Media(
                    creator=self.creator_name,
                    media_url=item['url'],
                    media_type=self.media_type,
                    likes=item['stats']['likeCount'],
                    hearts=item['stats']['heartCount'],
                    prompt=item['meta'].get('prompt') if item.get('meta') is not None else None,
                    model=item['meta'].get('Model').split('.')[0] if item.get('meta') is not None and
                                                                     item['meta'].get('Model') is not None else None,
                    author=item['username'],
                    tags=self.tags,
                    source='Civitai',
                    media_created_at=item['createdAt']
                )
                result.append(media)
        return result


def main():
    load_dotenv()

    civitai_image_loader = CivitaiCollector(creator_name='CivitAIDailyTopImage', media_type='image', period='Day',
                                            nsfw=None, limit=3)
    civitai_image_loader.run()
    civitai_hot_image_loader = CivitaiCollector(creator_name='CivitAIDailyTopHotImage', media_type='image',
                                                period='Day', nsfw='Mature', limit=3)
    civitai_hot_image_loader.run()


if __name__ == "__main__":
    main()



from typing import Optional, Union
from datetime import datetime

from ai_datahive.models import DataHiveBaseModel

from pydantic import HttpUrl


class TelegramMessage(DataHiveBaseModel):
    telegram_group_topic_fk: str
    content: Optional[str] = None
    media_content: Optional[str] = None
    media_url: Optional[HttpUrl] = None
    media_type: Optional[str] = None
    scheduled_for: Optional[Union[datetime, str]] = None
    sent_at: Optional[Union[datetime, str]] = None
    status: str



import os

from .custom_exceptions import NoArgumentError, GetRequestError, NoCategoryError
import warnings
import datetime as dt
import requests
import feedparser
from bs4 import BeautifulSoup, MarkupResemblesLocatorWarning

## @package Library.arxiv_lib
#  Small library for making requests to the arXiv and parsing the results.
#
#  This library provides some methods to send requests to the arXiv (using their API),
#  and to parse and format the results. The library includes methods to perform simple
#  searches on the arXiv, as well as to read the daily RSS feeds, and to parse and extract
#  the relevant information to be sent to the users.

"""
This class is used to load the data from the arXiv and to provide the data to the bot.

Inspired by https://github.com/carlosparaciari/ArXivBot
"""


## List of all the categories of the arXiv.
ALL_CATEGORIES = ['stat.AP', 'stat.CO', 'stat.ML', 'stat.ME', 'stat.OT', 'stat.TH', 'stat', 'q-fin.PR', 'q-fin.RM',
                  'q-fin.PM', 'q-fin.TR',
                  'q-fin.MF', 'q-fin.CP', 'q-fin.ST', 'q-fin.GN', 'q-fin.EC', 'q-fin', 'q-bio.BM', 'q-bio.GN',
                  'q-bio.MN', 'q-bio.SC', 'q-bio.CB', 'q-bio.NC',
                  'q-bio', 'q-bio.TO', 'q-bio.PE', 'q-bio.QM', 'q-bio.OT', 'cs.AI', 'cs.CL', 'cs.CC', 'cs.CE', 'cs.CG',
                  'cs.GT', 'cs.CV', 'cs.CY', 'cs.CR',
                  'cs.DS', 'cs.DB', 'cs.DL', 'cs.DM', 'cs.DC', 'cs.ET', 'cs.FL', 'cs.GL', 'cs.GR', 'cs.AR', 'cs.HC',
                  'cs.IR', 'cs.IT', 'cs.LG', 'cs.LO', 'cs.MS',
                  'cs.MA', 'cs.MM', 'cs.NI', 'cs.NE', 'cs.NA', 'cs.OS', 'cs.OH', 'cs.PF', 'cs.PL', 'cs.RO', 'cs.SI',
                  'cs.SE', 'cs.SD', 'cs.SC', 'cs.SY', 'cs',
                  'astro-ph.GA', 'astro-ph.CO', 'astro-ph.EP', 'astro-ph.HE', 'astro-ph.IM', 'astro-ph.SR', 'astro-ph',
                  'cond-mat.dis-nn', 'cond-mat.mtrl-sci',
                  'cond-mat.mes-hall', 'cond-mat.other', 'cond-mat.quant-gas', 'cond-mat.soft', 'cond-mat.stat-mech',
                  'cond-mat.str-el', 'cond-mat.supr-con',
                  'cond-mat', 'gr-qc', 'hep-ex', 'hep-lat', 'hep-ph', 'hep-th', 'math-ph', 'nlin.AO', 'nlin.CG',
                  'nlin.CD', 'nlin.SI', 'nlin.PS', 'nlin',
                  'nucl-ex', 'nucl-th', 'physics', 'physics.acc-ph', 'physics.app-ph', 'physics.ao-ph',
                  'physics.atom-ph', 'physics.atm-clus', 'physics.bio-ph',
                  'physics.chem-ph', 'physics.class-ph', 'physics.comp-ph', 'physics.data-an', 'physics.flu-dyn',
                  'physics.gen-ph', 'physics.geo-ph',
                  'physics.hist-ph', 'physics.ins-det', 'physics.med-ph', 'physics.optics', 'physics.ed-ph',
                  'physics.soc-ph', 'physics.plasm-ph',
                  'physics.pop-ph', 'physics.space-ph', 'econ', 'eess', 'quant-ph', 'math', 'math.AG', 'math.AT',
                  'math.AP', 'math.CT', 'math.CA', 'math.CO',
                  'math.AC', 'math.CV', 'math.DG', 'math.DS', 'math.FA', 'math.GM', 'math.GN', 'math.GT', 'math.GR',
                  'math.HO', 'math.IT', 'math.KT', 'math.LO',
                  'math.MP', 'math.MG', 'math.NT', 'math.NA', 'math.OA', 'math.OC', 'math.PR', 'math.QA', 'math.RT',
                  'math.RA', 'math.SP', 'math.ST', 'math.SG']

def number_categories():
    return len(ALL_CATEGORIES)


## This function takes an integer \f$i\f$ and returns the \f$i\f$-th category in the list ALL_CATEGORY.
#
#  @param category_index Integer number
def single_category(category_index):
    if not isinstance(category_index, int):
        raise TypeError('The index is not an integer.')

    if len(ALL_CATEGORIES) > category_index >= 0:
        return ALL_CATEGORIES[category_index]
    else:
        raise IndexError('The module-scope list ALL_CATEGORY has no element ' + str(category_index) + '.')


## This function reviews the dictionary obtained from parse_response and returns title, author and link for each entry.
#
#  Only title, author and link are passed since they will go to the Telegram Bot.
#  The output of this function is therefore a list of dictionary with these entries.
#  Two different behaviours are expected from this function, depending on the
#  value of feed_type, which can be API or RSS (the first is used for searches,
#  the second for new submissions)
#
#  @param dictionary This is the output of the function @ref parse_response
#  @param max_number_authors The maximum number of authors to be shown (then they are replaced by 'et al.')
#  @param feed_type The type of feed to review (can be API or RSS)
def review_response(dictionary, max_number_authors, feed_type):
    results_list = []

    if not isinstance(max_number_authors, int):
        raise TypeError('The number of authors has to be an integer.')

    if max_number_authors < 1:
        raise ValueError('The maximum number of authors has to be bigger than 1.')

    if not isinstance(dictionary, dict):
        raise TypeError('The argument passed is not a dictionary.')

    try:
        if not isinstance(dictionary['entries'], list):
            raise TypeError('The field entries is corrupted.')
    except KeyError:
        raise NoArgumentError('No entries have been found during the search.')

    for entry in dictionary['entries']:

        if not isinstance(entry, dict):
            raise TypeError('One of the entries is corrupted.')

        if feed_type == 'API':
            element = {'title': prepare_title_field_API(entry),
                       'authors': prepare_authors_field_API(entry, max_number_authors),
                       'date': find_publishing_date(entry)}
        elif feed_type == 'RSS':
            if is_update(entry):
                continue
            element = {'title': prepare_title_field_RSS(entry),
                       'authors': prepare_authors_field_RSS(entry, max_number_authors),
                       'id': is_field_there(entry, 'id'),
                       'rights': is_field_there(entry, 'rights'),
                       'description': is_field_there(entry, 'description')}
        else:
            raise ValueError('Wrong feed type. It can only be API or RSS.')

        element['link'] = is_field_there(entry, 'link')

        # Check whether all field in the element are None
        is_empty = element['title'] is None and element['authors'] is None and element['link'] is None

        if not is_empty:
            results_list.append(element)

    if len(results_list) == 0:
        raise NoArgumentError('No entries have been found during the search.')

    return results_list


## This function formats the title, and is needed for the review_response.
#
#  This function removes the newline symbols \n from the title, and escapes the
#  HTML symbols <, >, &, so that they are correctly interpreted by telepot.send_message().
#
#  @param dictionary This is the output of the function @ref parse_response
def prepare_title_field_API(dictionary):
    title = is_field_there(dictionary, 'title')

    if isinstance(title, str):
        title = title.replace(u'\n', u'')
        title = title.replace(u'  ', u' ')
        return title
    else:
        return None


## This function formats the authors, and is needed for the review_response.
#
#  This function unifies the name of the authors in a single one, and returns a Unicode string (if there are some authors).
#  It also cuts the number of authors after max_number_authors, and replaces the remaining with 'et al.'
#
#  @param dictionary This is the output of the function @ref parse_response
#  @param max_number_authors The maximum number of authors to be shown (then they are replaced by 'et al.')
def prepare_authors_field_API(dictionary, max_number_authors):
    authors_list = is_field_there(dictionary, 'authors')
    authors_string = ''
    authors_number = 1

    if isinstance(authors_list, list):
        for author in authors_list:
            author_name = is_field_there(author, 'name')
            if authors_number > max_number_authors:
                authors_string = authors_string + u'et al.**'
                break
            if isinstance(author_name, str) and authors_number <= max_number_authors:
                authors_number += 1
                authors_string = authors_string + author_name + ', '

    else:
        return None

    # Check if we have something in the authors string
    if len(authors_string) == 0:
        return None
    else:
        authors_string = authors_string[: -2]
        return authors_string


## This function prepares the title, and is needed for the review_response.
#
#  This function prepares the title field after receiving an entry from the RSS feed.
#  The main difference with the API search is the presence of the arXiv id, which is removed.
#
#  @param dictionary This is the output of the function @ref parse_response
def prepare_title_field_RSS(dictionary):
    title = prepare_title_field_API(dictionary)

    return title


## This function formats the authors, and is needed for the review_response.
#
#  This function prepares the authors field after receiving an entry from the RSS feed.
#  The function is different from the one used for standard search feeds as the authors
#  are given in a single line and hyper links are present. This function remove hyper links
#  and cut the number of authors if they are more than a maximum value
#
#  @param dictionary This is the output of the function @ref parse_response
#  @param max_number_authors The maximum number of authors to be shown (then they are replaced by 'et al.')
def prepare_authors_field_RSS(dictionary, max_number_authors):
    authors_string = is_field_there(dictionary, 'author')

    if isinstance(authors_string, str):
        end_string = authors_count_same_string(authors_string, max_number_authors)
        if end_string != -1:
            authors_string = authors_string[:end_string] + u', et al.'
        authors_string = remove_hyperlinks(authors_string)
        return authors_string
    else:
        return None


## This function returns the date of the RSS feed (as a datetime object).
#
#  This function looks in the dictionary prepared by @ref parse_response, and
#  finds the date in which the RSS feed was published.
#
#  @param dictionary This is the output of the function @ref parse_response
def find_date_RSS(dictionary):
    feed_details = is_field_there(dictionary, 'feed')
    feed_date = is_field_there(feed_details, 'updated')

    if feed_date is not None:
        if isinstance(feed_date, str) and len(feed_date) > 16:
            feed_date = feed_date[:16]
            feed_datetime = dt.datetime.strptime(feed_date, '%a, %d %b %Y')
            return feed_datetime
        else:
            raise TypeError('The argument passed is not a date string.')
    else:
        raise NoArgumentError('The RSS feed does not have the publication date.')


## This function removes hyper links, and is needed for the prepare_authors_field_RSS.
#
#  This function removes the hyper links (<a href = ***> ... </a>) from a string,
#  and return a unicode string.
#
#  @param string A string with hyper links inside
def remove_hyperlinks(string):
    warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)

    bs_string = BeautifulSoup(string, 'html.parser')

    for hlink in bs_string.findAll('a'):
        hlink.replaceWithChildren()

    return str(bs_string)


## This function finds where to put 'et al.' in the author string, and is needed for the prepare_authors_field_RSS.
#
#  This function finds the position of the "max_number_authors"-th comma in the string,
#  and returns it. If this comma doesn't exist, return -1.
#
#  @param authors_string A string with all the authors
#  @param max_number_authors The maximum number of authors to be shown (then they are replaced by 'et al.')
def authors_count_same_string(authors_string, max_number_authors):
    index = -1

    for iteration in range(max_number_authors):
        index = authors_string.find(',', index + 1)
        if index == -1:
            break

    return index


## This function checks whether the entry is new or updated, and is needed for the @ref review_response function.
#
#  This function checks if the entry is an update of a previous version of the paper.
#  Return True if it is, and False if is not. If the title field is absent, returns True.
#
#  @param dictionary This is the output of the function @ref parse_response
def is_update(dictionary):
    title = is_field_there(dictionary, 'title')

    if title is not None:
        index = title.find('UPDATED')
        if index == -1:
            return False

    return True


## This function returns the total number of results in the search.
#
#  @param dictionary This is the output of the function @ref parse_response
def total_number_results(dictionary):
    if not isinstance(dictionary, dict):
        raise TypeError('The argument passed is not a dictionary.')

    feed_information = is_field_there(dictionary, 'feed')

    if feed_information is None:
        raise NoArgumentError('No feed have been returned by the search.')

    if not isinstance(feed_information, dict):
        raise TypeError('The field feed is corrupted.')

    total_results = is_field_there(feed_information, 'opensearch_totalresults')

    if total_results is None:
        raise NoArgumentError('The feed got corrupted.')

    return int(total_results)


## This function checks if a key is inside a dictionary, and is needed for the @ref review_response function.
#
#  This function checks that the dictionary has something associated to the key, and if not, it returns None.
#
#  @param dictionary This is the output of the function @ref parse_response
#  @param key This is a key which might be in the dictionary
def is_field_there(dictionary, key):
    try:
        return dictionary[key]
    except:
        return None


## This function finds the publishing date of a given entry, and is needed for the @ref review_response function.
#
#  This function returns a datetime object.
#
#  @param dictionary This is the output of the function @ref parse_response
def find_publishing_date(dictionary):
    date = is_field_there(dictionary, 'published')

    if isinstance(date, str) and len(date) > 10:
        date = date[0:10]
        date = dt.datetime.strptime(date, '%Y-%m-%d')
        return date
    else:
        return None


## This function parses the output of the @ref request_to_arxiv function.
#
#  This function modifies the response obtained by the request library, making it a raw data (string).
#  Then, it parses the raw data using FeedParser, and returns a dictionary.
#
#  @param response This is the output of the function @ref request_to_arxiv
def parse_response(response):
    if not isinstance(response, requests.models.Response):
        raise TypeError('The argument passed is not a Response object.')

    rawdata = response.text

    parsed_response = feedparser.parse(rawdata)

    return parsed_response


## This function communicates with the arXiv and download the information.
#
#  @param arxiv_search_link The link to the arXiv website
def request_to_arxiv(arxiv_search_link):
    if not isinstance(arxiv_search_link, str):
        raise TypeError('The argument passed is not a string.')

    # Making a query to the arXiv
    try:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:122.0) Gecko/20100101 Firefox/122.0',
        }
        response = requests.get(arxiv_search_link, headers=headers)
    except requests.exceptions.InvalidSchema as invalid_schema:
        raise invalid_schema
    except requests.exceptions.MissingSchema as missing_schema:
        raise missing_schema
    except:
        raise GetRequestError('Get from arXiv failed. Might be connection problem. Used: ' + arxiv_search_link)

    # Check the status of the response
    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError as connection_error:
        raise connection_error
    else:
        return response


## This function adds to the arXiv link an extra field, which specifies the number of results we want to obtain.
#
#  **NOTE**: This function is not used any more, since @ref simple_search now takes care of this.
#
#  @param arxiv_search_link The link to the arXiv website
#  @param number_of_results An integer number
def specify_number_of_results(arxiv_search_link, number_of_results):
    if number_of_results < 0:
        raise ValueError('The number of results you are interested in cannot be negative.')

    arxiv_search_link += '&max_results=' + str(number_of_results)

    return arxiv_search_link


## This function adds a specific category to the arXiv RSS link.
#
#  @param subject_category A category of the arXiv
#  @param arxiv_search_link The link to the arXiv website
def search_day_submissions(subject_category, arxiv_search_link):
    if not category_exists(subject_category):
        raise NoCategoryError('The passed category is not in the ArXiv')

    arxiv_search_link += subject_category

    return arxiv_search_link


## This function checks whether a category exists.
#
#  @param subject_category A (possible) category of the arXiv
def category_exists(subject_category):
    return subject_category in ALL_CATEGORIES


## This function prepare the query for each field
#
#  The field query is composed by a key (for example, au: for author), one or more
#  values, and some connectors (AND, or OR). Values is a list of unicode strings, and
#  each string can be a single word or a sentence. In the latter case, quotes need to be used.
#
#  @param key The field of the query
#  @param values A list of strings
#  @param connector The logic connector (can be AND or OR)
#  @param quote The quotes are needed for strings with multiple words
def prepare_field_query(key, values, connector, quote=''):
    query_search_string = ''

    if isinstance(values, list):
        for value in values:
            if isinstance(value, str):
                query_search_string += key + quote + value + quote + connector
        query_search_string = query_search_string[: - len(connector)]

    return query_search_string


## This function performs a search the arXiv.
#
#  This function assembles the link for the request to arXiv, which will be passed
#  to the @ref request_to_arxiv function. The search is made using some keywords,
#  but additional options can be added. It is possible to add one or more authors,
#  part of the title, one or more categories where to search in, and the time
#  interval (in years).
#
#  @param keywords a list of keywords of the search
#  @param arxiv_search_link The link to the arXiv website
#  @param start_num The number of the initial result
#  @param max_num The maximum number of shown results
#  @param authors The authors list (optional)
#  @param title The title (optional)
#  @param categories The categories (optional)
#  @param interval The time interval (optional)
def simple_search(keywords, arxiv_search_link, start_num, max_num, authors=None, title=None, categories=None,
                  interval=None):
    if authors is None:
        authors = []
    if title is None:
        title = []
    if categories is None:
        categories = []
    if interval is None:
        interval = []

    con_AND = '+AND+'
    con_OR = '+OR+'
    brackets = ['%28', '%29']
    start_opt = '&start='
    max_opt = '&max_results='

    length_check = len(arxiv_search_link)

    query_fields = [{'key': 'all:', 'values': keywords, 'connector': con_AND, 'quotes': ''},
                    {'key': 'au:', 'values': authors, 'connector': con_AND, 'quotes': '%22'},
                    {'key': 'ti:', 'values': title, 'connector': con_AND, 'quotes': '%22'},
                    {'key': 'cat:', 'values': categories, 'connector': con_OR, 'quotes': ''}
                    ]

    for query_field in query_fields:
        query_string = prepare_field_query(query_field['key'],
                                           query_field['values'],
                                           query_field['connector'],
                                           query_field['quotes']
                                           )
        if len(query_string) != 0:
            if query_field['key'] == 'cat:':
                query_string = brackets[0] + query_string + brackets[1]
            arxiv_search_link += query_string + con_AND

    if len(arxiv_search_link) == length_check:
        raise NoArgumentError('No arguments have been provided to the search.')
    else:
        arxiv_search_link = arxiv_search_link[: - len(con_AND)]

    # Prepare interval as submittedDate:[initial+TO+final]

    arxiv_search_link += start_opt + str(start_num) + max_opt + str(max_num)

    return arxiv_search_link


def send_and_parse_request(search_link):
    try:
        search_response = request_to_arxiv(search_link)
    except TypeError as TE:
        print('The url got corrupted. Try again!')
        # self.save_known_error_log(chat_identity, TE)
        raise
    except GetRequestError as GRE:
        print('The search arguments are fine, but the search on the arXiv failed.')
        # self.save_known_error_log(chat_identity, GRE)
        raise
    except requests.exceptions.HTTPError as HTTPE:
        print('We are currently experiencing connection problems, sorry!')
        # self.save_known_error_log(chat_identity, HTTPE)
        raise

    try:
        search_dictionary = parse_response(search_response)
    except TypeError as TE:
        print('The result of the search got corrupted.')
        # self.save_known_error_log(chat_identity, TE)
        raise

    return search_dictionary


def do_today_search(arxiv_category=None, limit=None):
    if arxiv_category is None:
        arxiv_category = os.getenv('DEFAULT_CATEGORY')

    try:
        today_search_link = search_day_submissions(arxiv_category, os.getenv('ARXIV_RSS_LINK'))
    except NoCategoryError:
        print('Please use the arXiv subjects.\nSee http://arxitics.com/help/categories for further information.')
        # TODO Make a DB logging system self.save_unknown_error_log(chat_identity, 'arxiv_lib.search_day_submissions')
        return None

    search_list, feed_date = search_and_format_RSS(today_search_link)

    total_results = len(search_list)

    max_rss_result_number = int(os.getenv('MAX_RSS_RESULT_NUMBER')) if limit is None or limit < 0 else limit
    search_list = search_list[:max_rss_result_number]
    remaining_results = total_results - max_rss_result_number

    return send_results_back_rss(search_list, remaining_results, arxiv_category, feed_date)


def search_and_format_RSS(search_link):

    search_dictionary = send_and_parse_request(search_link)
    max_number_authors = int(os.getenv('MAX_NUMBER_AUTHORS'))

    try:
        search_list = review_response(search_dictionary, max_number_authors, 'RSS')
    except NoArgumentError:
        print('There are no submissions to your favourite category today, try tomorrow!')
        raise
    except TypeError as TE:
        print('The result of the search got corrupted.')
        # self.save_known_error_log(chat_identity, TE)
        raise
    except ValueError as VE:
        print('We are experiencing some technical problems, sorry!')
        # self.save_known_error_log(chat_identity, VE)
        raise

    try:
        feed_date = find_date_RSS(search_dictionary)
    except NoArgumentError as NAE:
        print('The result of the search got corrupted.')
        # self.save_known_error_log(chat_identity, NAE)
        raise
    except TypeError as TE:
        print('The result of the search got corrupted.')
        # self.save_known_error_log(chat_identity, TE)
        raise

    return search_list, feed_date


def check_size_and_split_message(message, new_item):

    message_would_exceed = len(message) + len(new_item) >= int(os.getenv('MAX_CHARACTERS_CHAT'))
    if message_would_exceed:
        message = ''
    message += new_item

    return message


def send_results_back_rss(search_list, remaining_results, arxiv_category, feed_date):

    result_counter = 1
    today = feed_date + dt.timedelta(days=1)
    message_result = 'List of submissions to <b>' + arxiv_category + '</b> for today ' + today.strftime(
        "%a, %d %b %y") + '.\n\n'

    message_result = []
    now = dt.datetime.now()

    for result in search_list:
        description = result['description'].split('Abstract: ')[1]
        new_item = {
            'title': result['title'],
            'authors': result['authors'],
            'description': description,
            'date': now.strftime("%d.%m.%Y"),
            'link': result['link'],
            'guid': result['id'],
            'rights': result['rights']
        }

        message_result.append(new_item)

        result_counter += 1

    if remaining_results > 0:
        remaining_information = {
            'title': 'Remaining Information',
            'description': f'There are {remaining_results} remaining submissions today. Consider visiting the '
                           f'arXiv web-page to see them.'
        }

        # Hinzufügen der verbleibenden Informationen als neues Item
        message_result.append(remaining_information)

    return message_result



from typing import List

from ai_datahive.utils.datetime_helper import today_as_start_and_enddate_str

from ai_datahive.transformers import BaseContentTransformer

from ai_datahive.collectors.models import ResearchPaper
from ai_datahive.transformers.models import Content

from ai_datahive.services import OpenAIService, PromptService


class DailyArxivPaperTransformer(BaseContentTransformer):
    def __init__(self, creator_name='DailyArxivPaper', template_file_name='daily_arxiv_paper_template.html', language='de'):
        from ai_datahive.dao.dao_factory import dao_factory
        self.dao = dao_factory()

        self.ps = PromptService()
        self.oais = OpenAIService()

        super().__init__(creator_name=creator_name, template_file_name=template_file_name, language=language)

    def retrieve(self) -> List[ResearchPaper]:
        start_date_str, end_date_str = today_as_start_and_enddate_str()

        filters = [["creator", "ArxivCollector"], ['created_at', 'between', start_date_str, end_date_str]]
        papers = self.dao.read(ResearchPaper, filters, limit=3, order_by='created_at')
        return papers

    def transform(self, papers: list[ResearchPaper]) -> List[Content]:
        # TODO make a language system. Getting the language from Paper and Check language in this transformer
        # TODO If different make a translation
        # Hardcoded for now
        paper_language = 'en'
        result = []

        for paper in papers:
            system_msg = self.ps.create_summarize_system_prompt()

            if self.language != paper_language:
                system_msg += self.ps.create_translate_system_prompt(language=self.language)

            self.oais.switch_text_model('gpt-3.5-turbo')
            abstract = self.oais.chat_response(system_msg, paper.abstract)
            self.oais.switch_to_default_text_model()

            template_data = {
                'title': paper.title,
                'abstract': abstract,
                'authors': paper.authors,
                'date': paper.created_at,
                'url': paper.paper_url
            }
            str_content = self.create_content(template_data)

            content = Content(
                title=paper.title,
                content=str_content,
                reference_url=paper.paper_url,
                reference_type='paper',
                reference_created_at=paper.created_at,
                source=paper.source,
                language=self.language,
                creator=self.creator_name,
                tags=paper.tags,
            )
            result.append(content)
        return result

    def run(self):
        # get top image
        # check if top image was already top image
        # if yes try next top image until three tries
        # If all already top images write a message with the first one to say it is again the winner. in a row.
        papers = self.retrieve()
        content = self.transform(papers)
        self.save(content)


def main():
    from dotenv import load_dotenv
    load_dotenv()

    transformer = DailyArxivPaperTransformer()
    transformer.run()


if __name__ == "__main__":
    main()



import os

from dotenv import load_dotenv
from ai_datahive.collectors import BaseCollector

from ai_datahive.collectors.utils.allowed_github_parameter import AllowedDateRanges
from ai_datahive.collectors.utils.scraping_helper import get_request
from ai_datahive.collectors.utils.github_helper import filter_articles, make_soup, scraping_repositories

from ai_datahive.collectors.models import GithubProject

from ai_datahive.utils import datetime_helper as dh


class GithubCollector(BaseCollector):

    VALID_PERIODS = ['Day', 'Week', 'Month']

    #TODO if needed use more parameters, right now fixed to github trending repos
    def __init__(self, creator_name='GithubCollector', period='Day', tags=None, limit=3):
        self.creator_name = creator_name
        self.limit = limit
        self.period = period
        self.media_type = 'github_project'
        self.github_url = os.getenv('GITHUB_TRENDS_URL')

        if tags is None:
            self.tags = self.build_tags(period)
        else:
            self.tags = tags

        self.validate_parameters(period)

        super().__init__(creator_name=self.creator_name, content_type=GithubProject)

    def validate_parameters(self, period):
        if period not in self.VALID_PERIODS:
            raise ValueError(f"period must be one of {self.VALID_PERIODS}")

    def build_tags(self, period):
        tags = 'github, '
        tags += 'trending, '
        tags += dh.to_periodic_format(period) + ', '

        tags = tags.rstrip(', ')
        return tags

    def trending_repositories(self, since: AllowedDateRanges = None):
        """Returns data about trending repositories (all programming
        languages, cannot be specified on this endpoint)."""
        if since:
            payload = {"since": since}
        else:
            payload = {"since": "weekly"}

        raw_html = get_request(self.github_url, payload)
        articles_html = filter_articles(raw_html)
        soup = make_soup(articles_html)
        return scraping_repositories(soup, since=payload[
            "since"])

    def retrieve(self):
        periodicity = dh.to_periodic_format(self.period)
        entries = self.trending_repositories(since=periodicity)
        projects = self.convert_to_github_project_entities(entries)
        return projects[:self.limit]

    def convert_to_github_project_entities(self, data):
        result = []
        if data:
            for item in data:

                contributors = [user['username'] for user in item['builtBy']]

                # Format output
                if len(contributors) > 3:
                    output = ', '.join(contributors[:3]) + ', et al.'
                else:
                    output = ', '.join(contributors)

                project = GithubProject(
                    creator=self.creator_name,
                    rank=item['rank'],
                    username=item['username'],
                    name=item['repositoryName'],
                    url=item['url'],
                    description=item['description'],
                    program_language=item['language'],
                    language=item['language'],
                    total_stars=item['totalStars'],
                    forks=item['forks'],
                    new_stars=item['starsSince'],
                    since=item['since'],
                    tags=self.tags,
                    source='GitHub Trending',
                    contributors=output
                )
                result.append(project)
        return result


def main():
    load_dotenv()
    github_collector = GithubCollector()
    data = github_collector.retrieve()
    result = github_collector.save(data)
    print(result)


if __name__ == "__main__":
    main()



import requests
from requests.exceptions import RequestException


def get_request(*args, **kwargs) -> str:
    """Synchronous GET request with requests."""
    try:
        response = requests.get(*args, **kwargs)
        response.raise_for_status()  # Löst eine Exception für HTTP Fehlercodes aus
        return response.text
    except RequestException as e:
        return str(e)  # Gibt die Fehlermeldung als String zurück


def get_request_as_json(*args, **kwargs) -> str:
    """Synchronous GET request with requests."""
    try:
        response = requests.get(*args, **kwargs)
        response.raise_for_status()  # Löst eine Exception für HTTP Fehlercodes aus
        return response.json()
    except RequestException as e:
        return str(e)  # Gibt die Fehlermeldung als String zurück
    



def escape_html(text):
    """
    Escapes HTML special characters in the given text.
    """
    if isinstance(text, str):
        return (text.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;')
            .replace("'", '&#39;'))
    else:
        return text


def replace_numbers_with_emojis(text):
    # Emoji-Zuordnungen für Zahlen 0 bis 9
    emoji_map = {
        '0': '0️⃣',
        '1': '1️⃣',
        '2': '2️⃣',
        '3': '3️⃣',
        '4': '4️⃣',
        '5': '5️⃣',
        '6': '6️⃣',
        '7': '7️⃣',
        '8': '8️⃣',
        '9': '9️⃣'
    }
    # Ersetze jede Zahl im Text durch das entsprechende Emoji
    for number, emoji in emoji_map.items():
        text = text.replace(number, emoji)
    return text



#  Micro-library implementing some exceptions used in @ref Library.arxiv_lib

## Exception raised when no arguments are passed to the function
class NoArgumentError(Exception):
	pass

## Exception raised when get from arXiv fails
class GetRequestError(Exception):
	pass

## Exception raised when some generic error is raised
class UnknownError(Exception):
	pass

## Exception raised when a category does not belong to the arXiv
class NoCategoryError(Exception):
	pass


from datetime import datetime
from pyzotero.zotero import Zotero


class ZoteroHelper:

    def __init__(self, zotero_api_key, library_id, library_type='user'):
        self.zotero_client = Zotero(library_id, library_type, zotero_api_key)

    def get_latest_entries(self, limit=5):
        return self.zotero_client.top(limit=limit)

    def get_latest_entries_from_collection(self, collection_id, limit=5):
        return self.zotero_client.collection_items(collection_id, limit=limit)

    def get_latest_entries_from_collection_by_name(self, collection_name, limit=5):
        collection_id = self.find_collection_by_path(collection_name)
        if collection_id:
            return self.get_primary_items_from_collection(collection_id, limit=limit)
        else:
            return []

    def get_latest_created_after_by_collection_name(self, date_str, collection_name, limit=5, all_types=False):
        collection_id = self.find_collection_by_path(collection_name)
        if collection_id:
            return self.get_entries_created_after(date_str, collection_id, limit=limit, all_types=all_types)
        else:
            return []

    def get_all_collection_ids(self, collection_id):
        """
        Rekursive Funktion, um die IDs aller SubCollections einer Collection zu sammeln.
        """
        collection_ids = [collection_id]
        subcollections = self.zotero_client.collections_sub(collection_id)
        for subcollection in subcollections:
            subcollection_id = subcollection['key']
            collection_ids.extend(self.get_all_collection_ids(subcollection_id))
        return collection_ids

    def get_entries_created_after(self, date, collection_id, limit=5, all_types=False):
        date_filter = date.isoformat()
        all_collection_ids = self.get_all_collection_ids(collection_id)

        all_items = []
        for cid in all_collection_ids:
            params = {
                'limit': limit,
                'sort': 'dateAdded',
                'direction': 'desc',
            }

            if not all_types:
                params['itemType'] = '-attachment || note'

            items = self.zotero_client.collection_items(cid, **params)
            all_items.extend(items)

        sorted_items = sorted(all_items, key=lambda x: x['data']['dateAdded'], reverse=True)

        filtered_items = [item for item in sorted_items if item['data']['dateAdded'] > date_filter]

        if limit is not None:
            filtered_items = filtered_items[:limit]

        return filtered_items

    def get_primary_items_from_collection(self, collection_id, limit=5):
        params = {
            'itemType': '-attachment || note',
            # Dies ist ein Beispiel und funktioniert möglicherweise nicht wie erwartet
            'limit': limit
        }
        #items = self.zotero_client.collection_items(collection_id, limit=limit, content='json')
        items = self.zotero_client.collection_items(collection_id, **params)
        primary_items = items
        #primary_items = [item for item in items if item['itemType'] not in ['attachment', 'note']]
        return primary_items

    def find_collection_by_path(self, path, parent_collection_id=None):
        # Bestimmen, welche Sammlungen abgefragt werden sollen
        if parent_collection_id:
            collections = self.zotero_client.collections_sub(parent_collection_id)
        else:
            collections = self.zotero_client.collections_top() if '/' in path else self.zotero_client.all_collections()

        segments = path.split('/')
        for collection in collections:
            if collection['data']['name'] == segments[0]:
                # Letztes Segment erreicht, gewünschte Sammlung gefunden
                if len(segments) == 1:
                    return collection['data']['key']
                # Rekursion mit der nächsten Ebene des Pfades
                return self.find_collection_by_path('/'.join(segments[1:]), collection['data']['key'])

        # Warnung, wenn das Segment nicht gefunden wurde, und Rückgabe von None
        print(f"Warnung: Sammlung '{segments[0]}' nicht gefunden.")
        return None


def main():
    import os
    from dotenv import load_dotenv
    load_dotenv()

    zoter_key = os.getenv('ZOTERO_API_KEY')
    zoter_user_id = os.getenv('ZOTERO_USER_ID')
    zotero_loader = ZoteroHelper(zoter_key, library_id=zoter_user_id)

    #entries = zotero_loader.get_latest_entries_from_collection_by_name('Papers/arxiv', limit=5)
    search_date = datetime.strptime('2024-02-15', '%Y-%m-%d')
    entries = zotero_loader.get_latest_created_after_by_collection_name(search_date, 'Papers/arxiv', limit=5)
    for entry in entries:
        print(entry['data']['title'])
        print(entry['data']['dateAdded'])


if __name__ == "__main__":
    main()



import os

from ai_datahive.dao import BaseDAO, SupabaseDAO


def dao_factory() -> BaseDAO:
    dao_impl = os.getenv('DAO_IMPL', 'supabase')
    if dao_impl == 'supabase':
        return SupabaseDAO()
    else:
        return SupabaseDAO()



from typing import Type, Union
from collections.abc import Collection

from datetime import datetime, timezone, timedelta

from ai_datahive.models import ContentBaseModel


class BaseCollector:

    def __init__(self, content_type: Type[ContentBaseModel], creator_name: str,
                 run_interval: timedelta = timedelta(days=1)):
        from ai_datahive.dao.dao_factory import dao_factory

        self.creator_name = creator_name
        self.content_type = content_type
        self.run_interval = run_interval

        self.dao = dao_factory()

    def get_latest_message_date(self):
        # Get date from the latest message to the telegram table - using topic / reporter
        result = self.dao.get_latest_entity_date(self.content_type, self.creator_name)
        return result

    def retrieve(self) -> Union[ContentBaseModel, Collection[ContentBaseModel]]:
        # Get data from the source
        raise NotImplementedError

    def save(self, data: Union[ContentBaseModel, Collection[ContentBaseModel]]):
        if isinstance(data, Collection):
            result = []
            for item in data:
                result.append(self.dao.create(item))
            return result
        else:
            return self.dao.create(data)

    def is_due(self):
        latest_entity_date = self.get_latest_message_date()
        if latest_entity_date is None:
            return True

        latest_entity_date = latest_entity_date.astimezone(timezone.utc)
        now = datetime.now(timezone.utc)
        time_diff = now - latest_entity_date

        if time_diff < self.run_interval:
            total_seconds = time_diff.total_seconds()
            hours = int(total_seconds // 3600)
            minutes = int((total_seconds % 3600) // 60)
            print(f"Weniger als 24 Stunden seit der letzten Nachricht vergangen: "
                  f"Vergangen bisher: {hours:02d}:{minutes:02d}")
            return False
        return True

    def run(self):
        if self.is_due():
            try:
                data = self.retrieve()
                self.save(data=data)
            except Exception as e:
                print(f"Error in {self.creator_name}: {e}")



from typing import Union, Optional
from dataclasses import field
from datetime import datetime
from pydantic import BaseModel


class DataHiveBaseModel(BaseModel):
    id: Optional[Union[str, int]] = None
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)



import os
from typing import List

from dotenv import load_dotenv
from datetime import datetime, timezone

from ai_datahive.collectors import BaseCollector
import ai_datahive.collectors.utils.arxiv_helper as ah

from ai_datahive.utils.datetime_helper import to_periodic_format
from ai_datahive.collectors.models import ResearchPaper


class ArxivCollector(BaseCollector):

    VALID_PERIODS = ['Day']

    def __init__(self, creator_name='ArxivCollector', arxiv_category='cs.AI', period='Day', tags=None, limit=3):
        self.validate_parameters(arxiv_category, period)

        self.creator_name = creator_name
        self.limit = limit
        self.arxiv_category = arxiv_category
        self.period = period

        if tags is None:
            self.tags = self.build_tags(arxiv_category, period)
        else:
            self.tags = tags

        super().__init__(creator_name=self.creator_name, content_type=ResearchPaper)

    def validate_parameters(self, arxiv_category, period) -> None:
        if arxiv_category not in ah.ALL_CATEGORIES:
            raise ValueError(f"category must be one of {ah.ALL_CATEGORIES}")
        if period not in self.VALID_PERIODS:
            raise ValueError(f"period must be one of {self.VALID_PERIODS}")

    def build_tags(self, arxiv_category, period) -> str:
        tags = 'arxiv, '
        tags += arxiv_category.lower() + ', '
        tags += to_periodic_format(period) + ', '

        tags = tags.rstrip(', ')
        return tags

    def retrieve(self) -> List[ResearchPaper]:
        entries = ah.do_today_search(self.arxiv_category, self.limit)
        papers = self.convert_to_research_paper_entities(entries)
        return papers

    def convert_to_research_paper_entities(self, data) -> List[ResearchPaper]:
        result = []
        if data:
            for item in data:
                if 'Remaining Information' in item['title']:
                    # TODO make here a news item??
                    #paper = ResearchPaper(
                    #    creator=self.creator_name,
                    #    title=item['title'],
                    #    description=item['description'],
                    #    tags=self.tags + ', count',
                    #    authors=self.creator_name,
                    #    source='Arxiv',
                        # TODO rework function call
                    #    reference_url=ah.search_day_submissions(self.arxiv_category, os.getenv('ARXIV_RSS_LINK'))
                    #)
                    pass
                else:
                    now = datetime.now(timezone.utc)
                    paper = ResearchPaper(
                        title=item['title'],
                        abstract=item['description'],
                        source_id=item['guid'],
                        authors=item['authors'],
                        license=item['rights'],
                        source='Arxiv',
                        creator=self.creator_name,
                        tags=self.tags,
                        paper_submitted_at=now,
                        paper_url=item['link'],
                        # TODO rework function call
                        reference_url=ah.search_day_submissions(self.arxiv_category, os.getenv('ARXIV_RSS_LINK'))
                    )
                    result.append(paper)
        return result


def main():
    load_dotenv()
    arxiv_collector = ArxivCollector()
    data = arxiv_collector.run()
    #result = arxiv_collector.save(data)
    print(data)
    #media = civitai_collector.convert_to_media(data)

    #print(media)


if __name__ == "__main__":
    main()



from typing import Any, Union, Type, List, Tuple, TypeVar
from datetime import datetime

from ai_datahive.models import DataHiveBaseModel

#TODO get rid of Any - it is str or List with another str elements.
Filter = Union[Tuple[str, Any], Tuple[str, Any, Any], Tuple[str, Any, Any, Any], List[Any]]
T = TypeVar('T', bound=DataHiveBaseModel)

class BaseDAO:
    def create(self, entity: DataHiveBaseModel) -> DataHiveBaseModel:
        raise NotImplementedError

    def read(self, entity: Type[T], filters: Filter = None, limit=None, order_by=None,
             order_dir='asc') -> list[T]:
        raise NotImplementedError

    def update(self, entity: DataHiveBaseModel, id=None) -> DataHiveBaseModel:
        raise NotImplementedError

    def delete(self, entity: DataHiveBaseModel, id=None):
        raise NotImplementedError

    def get_latest_entity_date(self, entity: Type[DataHiveBaseModel], creator_name: str) -> Union[datetime, None]:
        entity = self.read(entity, filters=[['creator', creator_name]], order_by='created_at', order_dir='desc',
                             limit=1)

        if entity:
            entity = entity[0]
            return entity.created_at
        else:
            return None



import os
from ai_datahive.services import BaseLLMService, BaseAIVisionService, OpenAIService


def load_llm_service(key) -> BaseLLMService:
    llm_service = os.getenv(key, 'openai')
    if llm_service == 'openai':
        return OpenAIService()
    else:
        return OpenAIService()


def load_ai_vision_service(key) -> BaseAIVisionService:
    ai_vision_service = os.getenv(key, 'openai')
    if ai_vision_service == 'openai':
        return OpenAIService()
    else:
        return OpenAIService()



from .collectors import BaseCollector, CivitaiCollector, GithubCollector, ArxivCollector, ZoteroCollector
from .dao import BaseDAO, T, Filter, dao_factory
from .models import DataHiveBaseModel, ContentBaseModel
from .services import PromptService, OpenAIService, AIBackedTranslationService, BaseLLMService, BaseAIVisionService
from .utils import datetime_helper, text_helper
from .transformers import BaseContentTransformer



from .base_collector import BaseCollector
from .civitai_collector import CivitaiCollector
from .github_collector import GithubCollector
from .arxiv_collector import ArxivCollector
from .zotero_collector import ZoteroCollector





# TODO supporting chats with memory
class BaseLLMService:

    def raw_chat_response(self, system_prompt, user_prompt):
        raise NotImplementedError

    def chat_response(self, system_prompt, user_prompt):
        raise NotImplementedError

    def switch_text_model(self, model_name):
        raise NotImplementedError

    def switch_to_default_text_model(self):
        raise NotImplementedError



from ai_datahive.utils.datetime_helper import today_as_start_and_enddate_str

from ai_datahive.transformers import BaseContentTransformer

from ai_datahive.collectors.models import Media
from ai_datahive.transformers.models import Content

from ai_datahive.services import OpenAIService, PromptService, AIBackedTranslationService


class TopDailyImageCritiqueTransformer(BaseContentTransformer):
    def __init__(self, creator_name='DailyImageCritiqueJS', template_file_name='top_daily_image_critique_template.html',
                 language='de'):
        from ai_datahive.dao.dao_factory import dao_factory
        self.dao = dao_factory()

        self.ps = PromptService()
        self.oais = OpenAIService()
        self.ts = AIBackedTranslationService()

        super().__init__(creator_name=creator_name, template_file_name=template_file_name, language=language)

    def retrieve(self) -> Media:
        start_date_str, end_date_str = today_as_start_and_enddate_str()

        filters = [["creator", "CivitAIDailyTopImage"], ['created_at', 'between', start_date_str, end_date_str]]
        media = self.dao.read(Media, filters, limit=1, order_by='likes')
        return media[0] if media else None

    def transform(self, topdailyimage: Media) -> Content:

        if topdailyimage:
            system_prompt = self.ps.create_system_prompt_from_file('js_style_image_critique_prompt.txt')

            user_prompt = f"Prompt: {topdailyimage.prompt}\nartist: {topdailyimage.author}"
            critique = self.oais.vision_response(system_prompt=system_prompt, user_prompt=user_prompt,
                                                 image_url=topdailyimage.media_url)
            if self.language != 'en':
                critique = self.ts.translate(critique, self.language)

            template_data = {
                'critique': critique,
                'author': topdailyimage.author,
                'created_at': topdailyimage.media_created_at,
                'media_url': topdailyimage.media_url,
            }
            str_content = self.create_content(template_data)

            content = Content(
                creator=self.creator_name,
                tags=topdailyimage.tags+', critique, JS style',
                title=f'Die Bildkritik mit Saltz',
                content=str_content,
                media_url=topdailyimage.media_url,
                media_type='image',
                media_created_at=topdailyimage.media_created_at,
                source=topdailyimage.source,
                language=self.language
            )
            return content


def main():
    from dotenv import load_dotenv
    load_dotenv()

    transformer = TopDailyImageCritiqueTransformer()
    transformer.run()


if __name__ == "__main__":
    main()



from typing import Dict
from typing import List
from typing import Any
from typing import Optional

import bs4


def filter_articles(raw_html: str) -> str:
    """Filters HTML out, which is not enclosed by article-tags.
    Beautifulsoup is inaccurate and slow when applied on a larger
    HTML string, this filtration fixes this.
    """
    raw_html_lst = raw_html.split("\n")

    # count number of article tags within the document (varies from 0 to 50):
    article_tags_count = 0
    tag = "article"
    for line in raw_html_lst:
        if tag in line:
            article_tags_count += 1

    # copy HTML enclosed by first and last article-tag:
    articles_arrays, is_article = [], False
    for line in raw_html_lst:
        if tag in line:
            article_tags_count -= 1
            is_article = True
        if is_article:
            articles_arrays.append(line)
        if not article_tags_count:
            is_article = False
    return "".join(articles_arrays)


def make_soup(articles_html: str) -> bs4.element.ResultSet:
    """HTML enclosed by article-tags is converted into a
    soup for further data extraction.
    """
    soup = bs4.BeautifulSoup(articles_html, "lxml")
    return soup.find_all("article", class_="Box-row")


def scraping_repositories(
    matches: bs4.element.ResultSet,
    since: str,
) -> List[Dict[Any, Any]]:
    """Data about all trending repositories are extracted."""
    trending_repositories = []
    for rank, match in enumerate(matches):

        # description
        if match.p:
            description = match.p.get_text(strip=True)
        else:
            description = None

        # relative url
        rel_url = match.h2.a["href"]

        # absolute url:
        repo_url = "https://github.com" + rel_url

        # name of repo
        repository_name = rel_url.split("/")[-1]

        # author (username):
        username = rel_url.split("/")[-2]

        # language and color
        progr_language = match.find("span", itemprop="programmingLanguage")
        if progr_language:
            language = progr_language.get_text(strip=True)
            lang_color_tag = match.find("span", class_="repo-language-color")
            lang_color = lang_color_tag["style"].split()[-1]
        else:
            lang_color, language = None, None

        stars_built_section = match.div.findNextSibling("div")

        # total stars:
        if stars_built_section.a:
            raw_total_stars = stars_built_section.a.get_text(strip=True)
            if "," in raw_total_stars:
                raw_total_stars = raw_total_stars.replace(",", "")
            if raw_total_stars:
                total_stars: Optional[int]
                try:
                    total_stars = int(raw_total_stars)
                except ValueError as missing_number:
                    print(missing_number)
            else:
                total_stars = None

        # forks
        if stars_built_section.a.findNextSibling("a"):
            raw_forks = stars_built_section.a.findNextSibling(
                "a",
            ).get_text(strip=True)
            if "," in raw_forks:
                raw_forks = raw_forks.replace(",", "")
        if raw_forks:
            forks: Optional[int]
            try:
                forks = int(raw_forks)
            except ValueError as missing_number:
                print(missing_number)
        else:
            forks = None

        # stars in period
        if stars_built_section.find(
                "span", class_="d-inline-block float-sm-right",
        ):
            raw_stars_since = (
                stars_built_section.find(
                    "span",
                    class_="d-inline-block float-sm-right",
                )
                .get_text(strip=True)
                .split()[0]
            )
            if "," in raw_stars_since:
                raw_stars_since = raw_stars_since.replace(",", "")
            if raw_stars_since:
                stars_since: Optional[int]
                try:
                    stars_since = int(raw_stars_since)
                except ValueError as missing_number:
                    print(missing_number)
            else:
                stars_since = None

        # builtby
        built_section = stars_built_section.find(
            "span",
            class_="d-inline-block mr-3",
        )
        if built_section:
            contributors = stars_built_section.find(
                "span",
                class_="d-inline-block mr-3",
            ).find_all("a")
            built_by = []
            for contributor in contributors:
                contr_data = {}
                contr_data["username"] = contributor["href"].strip("/")
                contr_data["url"] = "https://github.com" + contributor["href"]
                contr_data["avatar"] = contributor.img["src"]
                built_by.append(dict(contr_data))
        repositories = {
            "rank": rank + 1,
            "username": username,
            "repositoryName": repository_name,
            "url": repo_url,
            "description": description,
            "language": language,
            "languageColor": lang_color,
            "totalStars": total_stars,
            "forks": forks,
            "starsSince": stars_since,
            "since": since,
            "builtBy": built_by,
        }
        trending_repositories.append(repositories)
    return trending_repositories


def scraping_developers(
    matches: bs4.element.ResultSet,
    since: str,
) -> List[Dict[Any, Any]]:
    """Data about all trending developers are extracted."""
    all_trending_developers = []
    for rank, match in enumerate(matches):

        # relative url of developer
        rel_url = match.div.a["href"]

        # absolute url of developer
        dev_url = "https://github.com" + rel_url

        # username of developer
        username = rel_url.strip("/")

        # developers full name
        name = match.h1.a.get_text(strip=True) if match.h1.a else None

        # avatar url of developer
        avatar = match.img["src"] if match.img else None

        # data about developers popular repo:
        if match.article:
            raw_description = match.article.find(
                "div",
                class_="f6 color-text-secondary mt-1",
            )
            repo_description = (
                raw_description.get_text(
                    strip=True,
                )
                if raw_description
                else None
            )
            pop_repo = match.article.h1.a
            if pop_repo:
                repo_name = pop_repo.get_text(strip=True)
                repo_url = "https://github.com" + pop_repo["href"]
            else:
                repo_name = None
                repo_url = None
        else:
            repo_description = None
            repo_name = None
            repo_url = None

        one_developer = {
            "rank": rank + 1,
            "username": username,
            "name": name,
            "url": dev_url,
            "avatar": avatar,
            "since": since,
            "popularRepository": {
                "repositoryName": repo_name,
                "description": repo_description,
                "url": repo_url,
            },
        }
        all_trending_developers.append(one_developer)
    return all_trending_developers



import os
import re
import inflect

from datetime import datetime
from typing import Annotated, get_origin, get_args, Union, Type, List

from ai_datahive.dao import BaseDAO, T, Filter
from ai_datahive.transformers.models import Content
from supabase import create_client, Client

from pydantic_core import Url


class SupabaseDAO(BaseDAO):

    def __init__(self):
        supabase_url = os.getenv('SUPABASE_URL')
        supabase_api_key = os.getenv('SUPABASE_KEY')

        self.supabase_client: Client = create_client(supabase_url, supabase_api_key)

    def create(self, entity: T) -> T:
        table_name = self.table_name_from_model(type(entity))
        row_data = self._model_instance_to_row(entity)
        del row_data['id']
        response = self.supabase_client.table(table_name).insert(row_data).execute()
        data = response.data
        return self._row_to_model_instance(data[0], type(entity))

    def read(self, entity: Type[T], filters=None, limit=None, order_by=None, order_dir='asc') -> (
            Union)[Type[T], list[Type[T]]]:
        table_name = self.table_name_from_model(entity)
        result = self.supabase_client.table(table_name).select("*")

        if filters is not None:
            query_filters = self.convert_filters_to_query(filters)
            for query in query_filters:
                # Trennen des Feldnamens vom Rest der Bedingung
                field_operator_criteria = query.split('=', 1)
                if len(field_operator_criteria) == 2:
                    field, operator_criteria = field_operator_criteria
                    # Nun müssen wir operator_criteria in Operator und Kriterium aufteilen
                    operator, criteria = operator_criteria.split('.', 1)
                    result = result.filter(field, operator, criteria)
                else:
                    # Fallback, falls das Format ungültig ist
                    print(f"Fehler: Filterformat ist ungültig: '{query}'")

            # Limit anwenden, wenn spezifiziert
        if limit is not None:
            result = result.limit(limit)

            # Order By und Richtung anwenden, wenn spezifiziert
        if order_by is not None:
            result = result.order(order_by, desc=(order_dir == 'asc'))

        return [self._row_to_model_instance(row, entity) for row in result.execute().data]

    def update(self, entity: T, id=None):
        table_name = self.table_name_from_model(type(entity))
        row_data = self._model_instance_to_row(entity)
        if id is None:
            id = entity.id

        self.supabase_client.table(table_name).update(row_data).eq('id', id).execute()

    def delete(self, entity: T, id=None):
        # Implementiere die Methode unter Verwendung von Supabase
        pass

    def is_of_type(self, typ, check) -> bool:
        """Prüft, ob `typ` dem `check` Typ entspricht, auch innerhalb von Union- und Optional-Typen."""
        origin = get_origin(typ)
        if origin is Union:
            return any(self.is_of_type(arg, check) for arg in get_args(typ))
        elif origin is Annotated:
            # Prüfe den Basistyp in Annotated-Typen
            base_type = get_args(typ)[0]
            return self.is_of_type(base_type, check)
        elif origin:
            # Für andere generische Typen (wie List, Dict usw.)
            return False
        return isinstance(typ, type) and issubclass(typ, check)

    def convert_row_to_model_value(self, expected_type, value):
        """Konvertiert einen Wert in den erwarteten Typ, wenn möglich."""
        if value is None:
            return None
        if self.is_of_type(expected_type, datetime):
            return datetime.fromisoformat(value) if isinstance(value, str) else value
        elif self.is_of_type(expected_type, Url):
            # Da HttpUrl ein spezialisierter Url-Typ ist, prüfen wir auf Url oder UrlStr
            return str(value)
            # Ergänzen Sie hier Konvertierungen für weitere spezielle Typen
        return value

    def convert_model_to_row_value(self, field_type, value):
        """Konvertiert einen Wert in den erwarteten Typ, wenn möglich."""
        if value is None:
            return None
        if self.is_of_type(field_type, datetime) and not isinstance(value, str):
            return value.isoformat()
        elif self.is_of_type(field_type, Url):
            return str(value)
            # Ergänzen Sie hier Konvertierungen für weitere spezielle Typen
        return value

    def _row_to_model_instance(self, row: dict, model: Type[T]) -> T:
        init_values = {}
        for field_name, field_info in model.__fields__.items():
            field_value = row.get(field_name)
            field_type = field_info.annotation
            converted_value = self.convert_row_to_model_value(field_type, field_value)
            init_values[field_name] = converted_value
        return model(**init_values)

    def _model_instance_to_row(self, model_instance: T) -> dict:
        row_data = {}
        for field_name, field_info in model_instance.__fields__.items():
            value = getattr(model_instance, field_name, None)
            field_type = field_info.annotation
            row_data[field_name] = self.convert_model_to_row_value(field_type, value)
        return row_data

    def table_name_from_model(self, model: Type[T]) -> str:
        inflect.def_classical["names"] = False
        p = inflect.engine()
        model_name = model.__name__
        table_name_snake_case = re.sub(r'(?<!^)(?=[A-Z])', '_', model_name).lower()

        # TODO find a better way to handle pluralization
        if 'media' in table_name_snake_case:
            table_name_plural = 'media'
        else:
            table_name_plural = p.plural_noun(table_name_snake_case)

        table_name = f"t_{table_name_plural}"
        return table_name

    def convert_filters_to_query(self, filters: List[Filter]) -> List[str]:
        query_filters = []

        def process_filter(filter: Filter):
            if isinstance(filter, list) and filter[0] == 'or':
                # Process an 'or' condition
                or_filters = [process_filter(f).replace('=', '.') for f in filter[1:]]
                return f"or=({','.join(or_filters)})"
            elif len(filter) == 2:
                # Standardfall: Feld und Wert (impliziert "=" Operator)
                field, value = filter
                return f"{field}=eq.{value}"
            elif len(filter) == 3:
                # Spezialfall: Feld, Operator, Wert
                field, operator, value = filter
                return f"{field}={operator}.{value}"
            elif len(filter) == 4:
                # Erweiterter Fall für spezielle Logiken wie "between"
                field, op_start, start_value, end_value = filter
                if op_start.lower() == "between":
                    return [f"{field}=gte.{start_value}", f"{field}=lte.{end_value}"]

        for filter in filters:
            result = process_filter(filter)
            if isinstance(result, list):
                query_filters.extend(result)
            else:
                query_filters.append(result)

        return query_filters

    #def get_latest_data_date(self, creator_name: str, table_name: str):
    #    # Get date from the latest message to the telegram table - using topic / reporter
    #    #table_name = f't_{content_type}'
    #    result = self.supabase_client.table(table_name) \
    #        .select('created_at') \
    #        .eq('creator', creator_name) \
    #        .order('created_at', desc=True) \
    #        .limit(1) \
    #        .execute()

    #    if result.data:
            # Gehe davon aus, dass `created_at` das Datum beinhaltet
    #        return datetime.fromisoformat(result.data[0]['created_at'])
    #    else:
    #        return datetime.now() - timedelta(weeks=1)

    # Implementiere die restlichen Methoden entsprechend


def main():
    from dotenv import load_dotenv
    from ai_datahive.publishers.models import TelegramMessage

    load_dotenv()

    dao = SupabaseDAO()
    #print(dao.read(TelegramMessage, limit=1))

    content = Content(
        title="Test",
        content="Test",
        creator="Test",
        tags="Test",
        source="Test",
        language="de"
    )

    #print(dao.create(content))
    now = datetime.utcnow()
    tmsgs = dao.read(TelegramMessage, filters=[['or', ["scheduled_for", "lte", now.isoformat()], ["scheduled_for", "is", "null"]], ["sent_at", "is", "null"]],
                     order_by="created_at")
    print(tmsgs)


if __name__ == "__main__":
    main()





class BaseAIVisionService:

    def raw_vision_response(self, system_prompt, user_prompt, image_url):
        raise NotImplementedError

    def vision_response(self, system_prompt, user_prompt, image_url):
        raise NotImplementedError

    def switch_vision_model(self, model_name):
        raise NotImplementedError



from typing import Union, Optional
from ai_datahive.models import ContentBaseModel
from pydantic import HttpUrl
from datetime import datetime


class Content(ContentBaseModel):
    title: Optional[str] = None
    content: Optional[str] = None
    language: str # ISO 639-1
    media_url: Optional[HttpUrl] = None
    media_type: Optional[str] = None
    media_created_at: Optional[Union[datetime, str]] = None
    reference_url: Optional[HttpUrl] = None
    reference_type: Optional[str] = None
    reference_created_at: Optional[Union[datetime, str]] = None
    scheduled_for: Optional[Union[datetime, str]] = None        



import os
from dotenv import load_dotenv
from datetime import datetime, timedelta

from ai_datahive.models import Media
from ai_datahive.collectors import BaseCollector
from ai_datahive.collectors.utils.zotero_helper import ZoteroHelper


class ZoteroCollector(BaseCollector):

    def __init__(self, collection_name, content_type='media', creator_name='ZoteroCollector', zotero_api_key=None,
                 zotero_library_id=None, zotero_library_type='user', all_types=False, tags=None, limit=3):

        zotero_api_key = os.getenv('ZOTERO_API_KEY') if zotero_api_key is None else zotero_api_key
        zotero_library_id = os.getenv('ZOTERO_USER_ID') if zotero_library_id is None else zotero_library_id

        self.zotero_library_id = zotero_library_id
        self.collection_name = collection_name
        self.limit = limit
        self.all_types = all_types
        self.content_type = content_type
        self.zotero_helper = ZoteroHelper(zotero_api_key, zotero_library_id, zotero_library_type)

        super().__init__(creator_name=creator_name, content_type=content_type)

    def build_tags(self, itemType, repository):
        tags = 'zotero, '
        if itemType:
            tags += itemType + ', '
        if repository:
            tags += repository + ', '

        tags = tags.rstrip(', ')
        return tags

    def retrieve(self):
        if self.collection_name is None or self.limit is None:
            raise ValueError('collection_name and limit must be defined')

        # Get date from the latest message to the telegram table - using topic / reporter
        latest_message_date = self.get_latest_message_date()
        print(latest_message_date)
        if latest_message_date is None:
            latest_message_date = (datetime.now() - timedelta(days=7))

        # Get everything what is new in the defined collection after that date
        entries = (self.zotero_helper
                   .get_latest_created_after_by_collection_name(latest_message_date, self.collection_name,
                                                                limit=self.limit, all_types=self.all_types))
        return entries

    # TODO needs to be changed after a good way of using a multi media collector is found
    def convert_to_media(self, data):
        if data:
            if self.content_type == 'media':
                creators = data['data'].get('creators', [])
                full_names = [f"{creator['firstName']} {creator['lastName']}" for creator in creators]

                # Format output
                if len(full_names) > 3:
                    output_creators = ', '.join(full_names[:3]) + ', et al.'
                else:
                    output_creators = ', '.join(full_names)

                media = Media(
                    creator=self.creator_name,
                    media_type='paper',
                    title=data['data'].get('title', ''),
                    description=data['data'].get('abstractNote', ''),
                    author=output_creators,
                    tags=self.build_tags(data['data'].get('itemType'), data['data'].get('repository')),
                    source='Zotero - ' + self.zotero_library_id,
                    media_created_at=data['data'].get('date', ''),
                    reference_url=data['data'].get('url')
                )
                return media


def main():
    load_dotenv()
    zotero_collector = ZoteroCollector(collection_name='Papers/arxiv')
    data = zotero_collector.retrieve()
    #print(data)
    #trans = zotero_collector.convert_to_media(data[0])
    #print(trans)
    result = zotero_collector.save(data)
    print(result)


if __name__ == "__main__":
    main()



import os
import openai

from ai_datahive.services import BaseLLMService, BaseAIVisionService


class OpenAIService(BaseLLMService, BaseAIVisionService):

    ALLOWED_TEXT_MODELS = ['gpt-4-1106-preview', 'gpt-3.5-turbo']
    ALLOWED_VISION_MODELS = ['gpt-4-vision-preview']

    def __init__(self, openai_key=None, text_model='gpt-4-1106-preview', vision_model='gpt-4-vision-preview'):
        openai_key = openai_key or os.getenv('OPENAI_API_KEY')
        self._openai_client = openai.OpenAI(api_key=openai_key)
        self.text_model = text_model
        self.vision_model = vision_model

    def switch_text_model(self, model_name):
        self.text_model = model_name

    def switch_to_default_text_model(self):
        self.text_model = 'gpt-4-1106-preview'

    def switch_vision_model(self, vision_model_name):
        self.vision_model = vision_model_name

    def raw_chat_response(self, system_prompt, user_prompt):
        response = self._openai_client.chat.completions.create(
            model=self.text_model,
            messages=[
                {
                    "role": "system", "content": f"{system_prompt}",
                },
                {
                    "role": "user",
                    "content": f"{user_prompt}",
                }
            ],
            max_tokens=3000,
            temperature=0.7
        )
        return response.choices[0]

    def chat_response(self, system_prompt, user_prompt):
        response = self.raw_chat_response(system_prompt, user_prompt)
        return response.message.content

    def raw_vision_response(self, system_prompt, user_prompt, image_url):
        response = self._openai_client.chat.completions.create(
            model=self.vision_model,
            messages=[
                {
                    "role": "system", "content": f"{system_prompt}"
                },
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": f"{user_prompt}"},
                        {
                            "type": "image_url",
                            "image_url": {
                                "url": f"{image_url}",
                            },
                        },
                    ],
                }
            ],
            max_tokens=3000,
            temperature=0.7,
        )
        return response.choices[0]

    def vision_response(self, system_prompt, user_prompt, image_url):
        response = self.raw_vision_response(system_prompt, user_prompt, image_url)
        return response.message.content



import os
import logging
import requests
import base64

from io import BytesIO
from datetime import datetime
from dotenv import load_dotenv

from telegram import Update
from telegram.ext import Application, ContextTypes, CallbackContext

from ai_datahive.publishers.models import TelegramMessage, TelegramGroupTopic

from ai_datahive.dao.dao_factory import dao_factory
from ai_datahive.dao import BaseDAO


def setup_logging():
    logging.basicConfig(
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        level=logging.INFO
    )


async def start_command(update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
    await update.message.reply_text('Hello! This is the TelegramGroupNotifier.')


def create_telegram_application() -> Application:
    telegram_token = os.getenv("TELEGRAM_TOKEN")
    application = Application.builder().token(telegram_token).build()
    # application.add_handler(CommandHandler('start', start_command))
    return application


def get_new_messages(dao: BaseDAO) -> list[TelegramMessage]:
    now = datetime.utcnow()
    # Fetch messages where 'schedule_for' is null or less than the current time, and 'sent_at' is null.
    filters = [
        ["or", ["scheduled_for", "lte", now.isoformat()], ["scheduled_for", "is", "null"]],
        ["sent_at", "is", "null"]
    ]
    tmsgs = dao.read(TelegramMessage, filters=filters,
                     order_by="created_at")

    return tmsgs


async def process_and_send_new_messages(context: CallbackContext):
    chat_id = os.getenv("CHAT_ID")
    dao: BaseDAO = context.job.data['dao']

    new_messages = get_new_messages(dao)

    if new_messages:
        for message in new_messages:
            await send_message(dao, context, chat_id, message)
            update_message_status(dao, message)


async def send_message(dao: BaseDAO, context: CallbackContext, chat_id: str, message: TelegramMessage):
    def split_content(content, max_length):
        return [content[i:i + max_length] for i in range(0, len(content), max_length)]

    content = message.content

    parse_mode = 'HTML' if content and '<' in content and '>' in content else None
    media = await prepare_media(message)

    message_thread_id = get_message_thread_id_by_topic_name(dao, message.telegram_group_topic_fk)
    #message_thread_id = get_message_thread_id_by_topic_name(supabase_client, topic) # COmmeted because bad db design with string

    max_message_length = int(os.getenv("TELEGRAM_MAX_MESSAGE_LENGTH", 4096))
    max_caption_length = int(os.getenv("TELEGRAM_MAX_CAPTION_LENGTH", 1024))

    media_methods = {
        'image': context.bot.send_photo,
        'video': context.bot.send_video,
        'audio': context.bot.send_audio,
    }

    base_args = {'chat_id': chat_id, 'message_thread_id': message_thread_id, 'parse_mode': parse_mode}

    message_object = None
    if message.media_type in media_methods and media:
        if content:
            if len(content) > max_caption_length:
                caption = content[:max_caption_length]
                rest_content = content[max_caption_length:]
            else:
                caption = content
                rest_content = None
        else:
            caption = None
            rest_content = None

        # Ergänzen der spezifischen Argumente für Medientypen
        send_method = media_methods[message.media_type]
        media_args = {'caption': caption} if content else {}
        media_args.update(base_args)
        await send_method(photo=media if message.media_type == 'image' else media, **media_args)
        if rest_content:
            for part in split_content(rest_content, max_message_length):
                await context.bot.send_message(text=part, **base_args)
    elif content:
        # Senden einer Textnachricht, falls kein Medieninhalt vorhanden oder erforderlich ist
        for part in split_content(content, max_message_length):
            await context.bot.send_message(text=part, **base_args)
    else:
        # Für den Fall, dass weder Inhalt noch Medien vorhanden sind (kann erweitert werden, falls nötig)
        print("Kein Inhalt oder Medien zum Senden vorhanden.")



    # TODO make it cleaner
    #message_object = None

    #if message.media_type == 'image' and content and media:
    #    message_object = await context.bot.send_photo(chat_id=chat_id, photo=media, message_thread_id=message_thread_id,caption=content, parse_mode=parse_mode)
    #elif message.media_type == 'video' and content and media:
    #    message_object = await context.bot.send_video(chat_id=chat_id, video=media, message_thread_id=message_thread_id,caption=content, parse_mode=parse_mode)
    #elif message.media_type == 'audio' and content and media:
    #    message_object = await context.bot.send_audio(chat_id=chat_id, audio=media, message_thread_id=message_thread_id,caption=content, parse_mode=parse_mode)
    #elif content:
    #    message_object = await context.bot.send_message(chat_id=chat_id, message_thread_id=message_thread_id, text=content, parse_mode=parse_mode)
    #elif media:
    #    if message.media_type == 'image':
    #        message_object = await context.bot.send_photo(chat_id=chat_id, photo=media, message_thread_id=message_thread_id)
    #    elif message.media_type == 'video':
    #        message_object = await context.bot.send_video(chat_id=chat_id, video=media, message_thread_id=message_thread_id)
    #    elif message.media_type == 'audio':
    #        message_object = await context.bot.send_audio(chat_id=chat_id, audio=media, message_thread_id=message_thread_id)

    #TODO: Thinking about pinning the message
    #if message_object:
    #    await context.bot.pin_chat_message(chat_id=chat_id, message_id=message_object.message_id)


def get_message_thread_id_by_topic_name(dao: BaseDAO, telegram_group_topic_fk: str):
    tgt = dao.read(TelegramGroupTopic, filters=[["id", telegram_group_topic_fk]], limit=1)
    if tgt:
        return tgt[0].telegram_group_topic_id
    else:
        return None


async def prepare_media(message: TelegramMessage):
    media_content = message.media_content
    media_url = message.media_url

    if media_content:
        return BytesIO(base64.b64decode(media_content))
    elif media_url:
        response = requests.get(media_url)
        return BytesIO(response.content)
    return None


def update_message_status(dao: BaseDAO, message: TelegramMessage):
    now = datetime.utcnow()
    message.sent_at = now.isoformat()
    message.status = 'sent'
    dao.update(message)


def main():
    setup_logging()
    load_dotenv()

    dao: BaseDAO = dao_factory()
    telegram_application: Application = create_telegram_application()

    (telegram_application.job_queue.run_repeating(process_and_send_new_messages,
                                                  interval=10, first=0, data={'dao': dao}))

    print('Telegram Bot started ...')
    # Run the bot until the user presses Ctrl-C
    telegram_application.run_polling(allowed_updates=Update.ALL_TYPES)


if __name__ == "__main__":
    main()



from typing import Union, List

from ai_datahive.publishers.models import TelegramMessage, TelegramGroup, TelegramGroupTopic

from ai_datahive.transformers.models import Content


class TelegramBaseLoader:
    def __init__(self, telegram_group_name, telegram_group_topic_id):
        from ai_datahive.dao.dao_factory import dao_factory
        self.dao = dao_factory()

        tgr = self.dao.read(TelegramGroup, [["telegram_group_name", telegram_group_name]], limit=1)
        if not tgr:
            raise ValueError(f"Telegram-Gruppe '{telegram_group_name}' nicht gefunden.")

        self.telegram_group_uuid = tgr[0].id
        self.telegram_group_topic_id = telegram_group_topic_id

    def retrieve(self):
        raise NotImplementedError

    def find_group_topic_uuid(self):
        filters = [
            ["telegram_group_topic_id", self.telegram_group_topic_id],
            ["telegram_group_fk", self.telegram_group_uuid]
        ]

        group_topic = self.dao.read(TelegramGroupTopic, filters, limit=1)
        if group_topic:
            return group_topic[0].id
        else:
            return None

    def save_content_as_telegram_message(self, content: Union[Content, List[Content]]):
        group_topic_uuid = self.find_group_topic_uuid()
        if not group_topic_uuid:
            print("Keine entsprechende group_topic_id gefunden.")
            return

        if isinstance(content, list):
            str_content = '\n\n'.join([c.content for c in content])
            content = content[0]
        else:
            str_content = content.content

        # Datenstruktur für die neue Nachricht
        message_data = {
            'telegram_group_topic_fk': group_topic_uuid,
            'content': f"<b>{content.title}</b>\n\n{str_content}",
            'creator': content.creator,
            'status': 'planned'
        }

        if content.media_type and (content.media_url or content.media_content):
            message_data['media_type'] = content.media_type
            if content.media_url:
                message_data['media_url'] = content.media_url
            elif content.media_content:
                message_data['media_content'] = content.media_content

        # Füge `schedule_for` hinzu, wenn ein Zeitpunkt angegeben ist
        if content.scheduled_for:
            message_data['schedule_for'] = content.schedule_for

        # Speichern der Nachricht in der Tabelle `t_telegram_messages`
        result = self.dao.create(TelegramMessage(**message_data))

    def load(self):
        content = self.retrieve()
        self.save_content_as_telegram_message(content)



from datetime import datetime
from typing import Union, Optional, ClassVar
from ai_datahive.models import ContentBaseModel
from pydantic import HttpUrl, field_validator
import base64


class Media(ContentBaseModel):
    VALID_MEDIA_TYPES: ClassVar[list[str]] = ['image', 'video', 'model', 'text', 'paper', 'news', 'prompt',
                                              'website', 'pdf']

    media_url: Optional[HttpUrl] = None
    media_b64_content: Optional[str] = None
    media_type: str
    title: Optional[str] = None
    description: Optional[str] = None
    likes: int = 0
    hearts: int = 0
    prompt: Optional[str] = None
    model: Optional[str] = None
    author: Optional[str] = None
    source: Optional[str] = None
    reference_url: Optional[HttpUrl] = None
    media_reference_id: Optional[Union['Media', str]] = None  # TODO To low leve, should refernce the media object
    media_created_at: Union[datetime, str]

    class Config:
        json_encoders = {
            HttpUrl: lambda v: str(v),
            datetime: lambda v: v.isoformat(),
        }

    @field_validator('media_b64_content')
    def check_base64_content(cls, v: str) -> Union[str, None]:
        if v is None:
            return v
        try:
            base64.b64decode(v)
            return v
        except ValueError:
            raise ValueError('media_b64_content must be base64 encoded')

    @field_validator('media_type')
    def check_media_type(cls, v: str) -> str:
        if v not in Media.VALID_MEDIA_TYPES:
            raise ValueError('media_type must be one of image, audio, video')
        return v

    @field_validator('title', 'description', 'author', mode='after')
    def check_strings(cls, v: str) -> Union[str, None]:
        if v is not None and not isinstance(v, str):
            raise ValueError('Field must be a string')
        return v

    @field_validator('likes', 'hearts', mode='after')
    def check_integers(cls, v: int) -> int:
        if not isinstance(v, int):
            raise ValueError('Field must be an integer')
        return v

    @field_validator('media_created_at', mode='before')
    def parse_image_created_at(cls, v: Union[datetime, str]) -> datetime:
        if isinstance(v, str):
            try:
                return datetime.fromisoformat(v)
            except ValueError:
                raise ValueError('image_created_at must be a valid datetime string or a datetime object')
        elif not isinstance(v, datetime):
            raise ValueError('image_created_at must be a datetime object')
        return v



from datetime import datetime
from typing import Union, Optional
from ai_datahive.models import ContentBaseModel
from pydantic import HttpUrl


class ResearchPaper(ContentBaseModel):
    title: str
    abstract: Optional[str] = None
    source_id: str
    authors: str
    license: Optional[str] = None
    paper_url: Optional[HttpUrl] = None
    code_url: Optional[HttpUrl] = None
    media_url: Optional[HttpUrl] = None
    reference_url: Optional[HttpUrl] = None
    paper_submitted_at: Union[datetime, str]



from .base_llm_service import BaseLLMService
from .base_ai_vision_service import BaseAIVisionService

from .openai_service import OpenAIService
from .prompt_service import PromptService

from .ai_backed_translation_service import AIBackedTranslationService




from ai_datahive.models import DataHiveBaseModel


class TelegramGroupTopic(DataHiveBaseModel):
    id: str
    telegram_group_fk: str
    telegram_group_topic_name: str
    telegram_group_topic_id: int



from ai_datahive.models import ContentBaseModel

from typing import Optional, ClassVar
from pydantic import HttpUrl, Field, field_validator


class GithubProject(ContentBaseModel):
    VALID_PERIODICITY: ClassVar[list[str]] = ['daily', 'weekly', 'monthly']

    rank: int
    username: str = Field(..., max_length=255)
    name: str = Field(..., max_length=255)
    url: HttpUrl
    description: Optional[str] = None
    program_language: Optional[str] = Field(None, max_length=255)
    total_stars: Optional[int] = None
    forks: Optional[int] = None
    new_stars: Optional[int] = None
    since: str = Field(..., max_length=50)
    contributors: Optional[str] = None

    class Config:
        json_encoders = {
            HttpUrl: lambda v: str(v)
        }

    @field_validator('since')
    def check_since(cls, v: str) -> str:
        if v not in cls.VALID_PERIODICITY:
            raise ValueError(f"since must be one of {cls.VALID_PERIODICITY}")
        return v



from datetime import datetime, timedelta


def to_periodic_format(period):
    if period == 'Day':
        return 'daily'
    elif period == 'Week':
        return 'weekly'
    elif period == 'Month':
        return 'monthly'
    elif period == 'Year':
        return 'yearly'
    elif period == 'AllTime':
        return 'all time'


def today_as_start_and_enddate_str(pattern='%Y-%m-%d'):
    today = datetime.now().date()
    start_date = today
    end_date = today + timedelta(days=1)
    start_date_str = start_date.strftime(pattern)
    end_date_str = end_date.strftime(pattern)

    return start_date_str, end_date_str



from ai_datahive.transformers.models import Content

from ai_datahive.utils.datetime_helper import today_as_start_and_enddate_str

from ai_datahive.publishers.loader import TelegramBaseLoader


class TelegramTopDailyImageCritiqueLoader(TelegramBaseLoader):

    def __init__(self, telegram_group_name, telegram_group_topic_id):
        super().__init__(telegram_group_name, telegram_group_topic_id)

    def retrieve(self):
        start_date_str, end_date_str = today_as_start_and_enddate_str()
        filters = [
            ["creator", "DailyImageCritiqueJS"],
            ["language", "de"],
            ["created_at", "between", start_date_str, end_date_str]
        ]

        topimage = self.dao.read(Content, filters, limit=1)
        return topimage[0]


def main():
    from dotenv import load_dotenv
    load_dotenv()
    IMAGE_GROUP_TOPIC_ID = 27
    loader = TelegramTopDailyImageCritiqueLoader('KI & Business', IMAGE_GROUP_TOPIC_ID)
    loader.load()


if __name__ == '__main__':
    main()



import os
from typing import Union, Collection

from string import Template
from ai_datahive.utils.text_helper import escape_html

from ai_datahive.transformers.models import Content


class BaseContentTransformer:
    def __init__(self, creator_name, template_file_name, language):
        from ai_datahive.dao.dao_factory import dao_factory
        self.dao = dao_factory()
        self.language = language
        self.creator_name = creator_name
        self.template_path = os.path.join(os.path.dirname(__file__), 'templates', template_file_name)

    def save(self, content: Union[Content, Collection[Content]]):
        if isinstance(content, Collection):
            results = [self.dao.create(c) for c in content]
            return results
        else:
            result = self.dao.create(content)
            return result

    def create_content(self, template_data):
        template_data = {k: escape_html(v) for k, v in template_data.items()}

        with open(self.template_path, 'r', encoding='utf-8') as file:
            template_content = file.read()

        template = Template(template_content)
        return template.safe_substitute(template_data)

    def retrieve(self):
        raise NotImplementedError

    def transform(self, data):
        raise NotImplementedError

    def run(self):
        # get top image
        # check if top image was already top image
        # if yes try next top image until three tries
        # If all already top images write a message with the first one to say it is again the winner. in a row.
        entities = self.retrieve()
        content = self.transform(entities)
        self.save(content)



from typing import List

from ai_datahive.collectors.models import GithubProject
from ai_datahive.transformers import BaseContentTransformer
from ai_datahive.transformers.models import Content

from ai_datahive.services import AIBackedTranslationService

from ai_datahive.utils.datetime_helper import today_as_start_and_enddate_str
from ai_datahive.utils.text_helper import replace_numbers_with_emojis

class DailyTrendingGithubProjectTransformer(BaseContentTransformer):
    def __init__(self, creator_name='DailyTrendingGithubProjects',
                 template_file_name='github_loader_trending_template.html', language='de'):
        from ai_datahive.dao.dao_factory import dao_factory
        self.dao = dao_factory()

        self.ts = AIBackedTranslationService()

        super().__init__(creator_name=creator_name, template_file_name=template_file_name, language=language)

    def retrieve(self) -> List[GithubProject]:
        start_date_str, end_date_str = today_as_start_and_enddate_str()

        filters = [["creator", "GithubCollector"], ['created_at', 'between', start_date_str, end_date_str]]
        projects = self.dao.read(GithubProject, filters, limit=3, order_by='created_at')
        return projects

    def transform(self, projects: list[GithubProject]) -> List[Content]:
        # TODO make a language system. Getting the language from Paper and Check language in this transformer
        # TODO If different make a translation
        # Hardcoded for now
        project_language = 'en'
        result = []

        for project in projects:
            if self.language != project_language:
                translated_description = self.ts.translate(project.description, self.language)
                project.description = translated_description

            template_data = {
                'rank': replace_numbers_with_emojis(str(project.rank)),
                'username': project.username,
                'repositoryName': project.name,
                'url': project.url,
                'description': project.description,
                'stars': project.total_stars,
                'starsSince': project.new_stars,
                'forks': project.forks,
            }
            str_content = self.create_content(template_data)

            content = Content(
                title=project.name,
                content=str_content,
                reference_url=project.url,
                reference_type='github_project',
                reference_created_at=project.created_at,
                source=project.source,
                language=self.language,
                creator=self.creator_name,
                tags=project.tags,
            )
            result.append(content)
        return result


def main():
    from dotenv import load_dotenv
    load_dotenv()

    transformer = DailyTrendingGithubProjectTransformer()
    transformer.run()


if __name__ == "__main__":
    main()



"""Allowed Parameters
===================
Path and query parameters which are allowed when client requests this API.
"""
# Copyright (c) 2021, Niklas Tiede.
# All rights reserved. Distributed under the MIT License.
from enum import Enum


class AllowedDateRanges(str, Enum):
    """Optional query parameter, default date range: daily"""

    daily = "daily"
    weekly = "weekly"
    monthly = "monthly"


class AllowedSpokenLanguages(str, Enum):
    """Optional query parameter, default language: any
    identifier (language name) = 2-char-string (abbrev. for urlParam)
    """

    Abkhazian = "ab"
    Afar = "aa"
    Afrikaans = "af"
    Akan = "ak"
    Albanian = "sq"
    Amharic = "am"
    Arabic = "ar"
    Aragonese = "an"
    Armenian = "hy"
    Assamese = "as"
    Avaric = "av"
    Avestan = "ae"
    Aymara = "ay"
    Azerbaijani = "az"
    Bambara = "bm"
    Bashkir = "ba"
    Basque = "eu"
    Belarusian = "be"
    Bengali = "bn"
    Bihari_languages = "bh"
    Bislama = "bi"
    Bosnian = "bs"
    Breton = "br"
    Bulgarian = "bg"
    Burmese = "my"
    Catalan_Valencian = "ca"
    Chamorro = "ch"
    Chechen = "ce"
    Chichewa_Chewa_Nyanja = "ny"
    Chinese = "zh"
    Chuvash = "cv"
    Cornish = "kw"
    Corsican = "co"
    Cree = "cr"
    Croatian = "hr"
    Czech = "cs"
    Danish = "da"
    Divehi_Dhivehi_Maldivian = "dv"
    Dutch_Flemish = "nl"
    Dzongkha = "dz"
    English = "en"
    Esperanto = "eo"
    Estonian = "et"
    Ewe = "ee"
    Faroese = "fo"
    Fijian = "fj"
    Finnish = "fi"
    French = "fr"
    Fulah = "ff"
    Galician = "gl"
    Georgian = "ka"
    German = "de"
    Greek_Modern = "el"
    Guarani = "gn"
    Gujarati = "gu"
    Haitian_Haitian_Creole = "ht"
    Hausa = "ha"
    Hebrew = "he"
    Herero = "hz"
    Hindi = "hi"
    Hiri_Motu = "ho"
    Hungarian = "hu"
    Interlingua_International_Auxil = "ia"
    Indonesian = "id"
    Interlingue_Occidental = "ie"
    Irish = "ga"
    Igbo = "ig"
    Inupiaq = "ik"
    Ido = "io"
    Icelandic = "is"
    Italian = "it"
    Inuktitut = "iu"
    Japanese = "ja"
    Javanese = "jv"
    Kalaallisut_Greenlandic = "kl"
    Kannada = "kn"
    Kanuri = "kr"
    Kashmiri = "ks"
    Kazakh = "kk"
    Central_Khmer = "km"
    Kikuyu_Gikuyu = "ki"
    Kinyarwanda = "rw"
    Kirghiz_Kyrgyz = "ky"
    Komi = "kv"
    Kongo = "kg"
    Korean = "ko"
    Kurdish = "ku"
    Kuanyama_Kwanyama = "kj"
    Latin = "la"
    Luxembourgish_Letzeburgesch = "lb"
    Ganda = "lg"
    Limburgan_Limburger_Limburgish = "li"
    Lingala = "ln"
    Lao = "lo"
    Lithuanian = "lt"
    Luba_Katanga = "lu"
    Latvian = "lv"
    Manx = "gv"
    Macedonian = "mk"
    Malagasy = "mg"
    Malay = "ms"
    Malayalam = "ml"
    Maltese = "mt"
    Maori = "mi"
    Marathi = "mr"
    Marshallese = "mh"
    Mongolian = "mn"
    Nauru = "na"
    Navajo_Navaho = "nv"
    North_Ndebele = "nd"
    Nepali = "ne"
    Ndonga = "ng"
    Norwegian_Bokmål = "nb"
    Norwegian_Nynorsk = "nn"
    Norwegian = "no"
    Sichuan_Yi_Nuosu = "ii"
    South_Ndebele = "nr"
    Occitan = "oc"
    Ojibwa = "oj"
    Church_Slavic_Old_Slavonic_Chu = "cu"
    Oromo = "om"
    Oriya = "or"
    Ossetian_Ossetic = "os"
    Punjabi_Panjabi = "pa"
    Pali = "pi"
    Persian = "fa"
    Polish = "pl"
    Pashto_Pushto = "ps"
    Portuguese = "pt"
    Quechua = "qu"
    Romansh = "rm"
    Rundi = "rn"
    Romanian_Moldavian_Moldovan = "ro"
    Russian = "ru"
    Sanskrit = "sa"
    Sardinian = "sc"
    Sindhi = "sd"
    Northern_Sami = "se"
    Samoan = "sm"
    Sango = "sg"
    Serbian = "sr"
    Gaelic_Scottish_Gaelic = "gd"
    Shona = "sn"
    Sinhala_Sinhalese = "si"
    Slovak = "sk"
    Slovenian = "sl"
    Somali = "so"
    Southern_Sotho = "st"
    Spanish_Castilian = "es"
    Sundanese = "su"
    Swahili = "sw"
    Swati = "ss"
    Swedish = "sv"
    Tamil = "ta"
    Telugu = "te"
    Tajik = "tg"
    Thai = "th"
    Tigrinya = "ti"
    Tibetan = "bo"
    Turkmen = "tk"
    Tagalog = "tl"
    Tswana = "tn"
    Tonga_Tonga_Islands = "to"
    Turkish = "tr"
    Tsonga = "ts"
    Tatar = "tt"
    Twi = "tw"
    Tahitian = "ty"
    Uighur_Uyghur = "ug"
    Ukrainian = "uk"
    Urdu = "ur"
    Uzbek = "uz"
    Venda = "ve"
    Vietnamese = "vi"
    Volapük = "vo"
    Walloon = "wa"
    Welsh = "cy"
    Wolof = "wo"
    Western_Frisian = "fy"
    Xhosa = "xh"
    Yiddish = "yi"
    Yoruba = "yo"
    Zhuang_Chuang = "za"
    Zulu = "zu"


class AllowedProgrammingLanguages(str, Enum):
    """Path Parameter
    identifier (language name) = string (urlParam)
    """

    c__ = "c++"
    html = "html"
    javascript = "javascript"
    jupyter_notebook = "jupyter-notebook"
    python = "python"
    shell = "shell"
    typescript = "typescript"
    unknown = "unknown"
    _1c_enterprise = "1c-enterprise"
    _4d = "4d"
    abap = "abap"
    abap_cds = "abap-cds"
    abnf = "abnf"
    actionscript = "actionscript"
    ada = "ada"
    adobe_font_metrics = "adobe-font-metrics"
    agda = "agda"
    ags_script = "ags-script"
    al = "al"
    alloy = "alloy"
    alpine_abuild = "alpine-abuild"
    altium_designer = "altium-designer"
    ampl = "ampl"
    angelscript = "angelscript"
    ant_build_system = "ant-build-system"
    antlr = "antlr"
    apacheconf = "apacheconf"
    apex = "apex"
    api_blueprint = "api-blueprint"
    apl = "apl"
    apollo_guidance_computer = "apollo-guidance-computer"
    applescript = "applescript"
    arc = "arc"
    asciidoc = "asciidoc"
    asl = "asl"
    asn1 = "asn.1"
    classic_asp = "classic-asp"
    aspnet = "asp.net"
    aspectj = "aspectj"
    assembly = "assembly"
    asymptote = "asymptote"
    ats = "ats"
    augeas = "augeas"
    autohotkey = "autohotkey"
    autoit = "autoit"
    avro_idl = "avro-idl"
    awk = "awk"
    ballerina = "ballerina"
    batchfile = "batchfile"
    beef = "beef"
    befunge = "befunge"
    bibtex = "bibtex"
    bison = "bison"
    bitbake = "bitbake"
    blade = "blade"
    blitzbasic = "blitzbasic"
    blitzmax = "blitzmax"
    bluespec = "bluespec"
    boo = "boo"
    boogie = "boogie"
    brainfuck = "brainfuck"
    brightscript = "brightscript"
    c = "c"
    c_23 = "c%23"
    c_objdump = "c-objdump"
    c2hs_haskell = "c2hs-haskell"
    cabal_config = "cabal-config"
    capn_proto = "cap'n-proto"
    cartocss = "cartocss"
    ceylon = "ceylon"
    chapel = "chapel"
    charity = "charity"
    chuck = "chuck"
    cirru = "cirru"
    clarion = "clarion"
    clean = "clean"
    click = "click"
    clips = "clips"
    clojure = "clojure"
    closure_templates = "closure-templates"
    cloud_firestore_security_rules = "cloud-firestore-security-rules"
    cmake = "cmake"
    cobol = "cobol"
    codeql = "codeql"
    coffeescript = "coffeescript"
    coldfusion = "coldfusion"
    coldfusion_cfc = "coldfusion-cfc"
    collada = "collada"
    common_lisp = "common-lisp"
    common_workflow_language = "common-workflow-language"
    component_pascal = "component-pascal"
    conll_u = "conll-u"
    cool = "cool"
    coq = "coq"
    cpp_objdump = "cpp-objdump"
    creole = "creole"
    crystal = "crystal"
    cson = "cson"
    csound = "csound"
    csound_document = "csound-document"
    csound_score = "csound-score"
    css = "css"
    csv = "csv"
    cuda = "cuda"
    curl_config = "curl-config"
    cweb = "cweb"
    cycript = "cycript"
    cython = "cython"
    d = "d"
    d_objdump = "d-objdump"
    dafny = "dafny"
    darcs_patch = "darcs-patch"
    dart = "dart"
    dataweave = "dataweave"
    desktop = "desktop"
    dhall = "dhall"
    diff = "diff"
    digital_command_language = "digital-command-language"
    dircolors = "dircolors"
    directx_3d_file = "directx-3d-file"
    dm = "dm"
    dns_zone = "dns-zone"
    dockerfile = "dockerfile"
    dogescript = "dogescript"
    dtrace = "dtrace"
    dylan = "dylan"
    e = "e"
    eagle = "eagle"
    easybuild = "easybuild"
    ebnf = "ebnf"
    ec = "ec"
    ecere_projects = "ecere-projects"
    ecl = "ecl"
    eclipse = "eclipse"
    editorconfig = "editorconfig"
    edje_data_collection = "edje-data-collection"
    edn = "edn"
    eiffel = "eiffel"
    ejs = "ejs"
    elixir = "elixir"
    elm = "elm"
    emacs_lisp = "emacs-lisp"
    emberscript = "emberscript"
    eml = "eml"
    eq = "eq"
    erlang = "erlang"
    f_23 = "f%23"
    f_ = "f*"
    factor = "factor"
    fancy = "fancy"
    fantom = "fantom"
    faust = "faust"
    figlet_font = "figlet-font"
    filebench_wml = "filebench-wml"
    filterscript = "filterscript"
    fish = "fish"
    flux = "flux"
    formatted = "formatted"
    forth = "forth"
    fortran = "fortran"
    fortran_free_form = "fortran-free-form"
    freemarker = "freemarker"
    frege = "frege"
    futhark = "futhark"
    g_code = "g-code"
    game_maker_language = "game-maker-language"
    gaml = "gaml"
    gams = "gams"
    gap = "gap"
    gcc_machine_description = "gcc-machine-description"
    gdb = "gdb"
    gdscript = "gdscript"
    gedcom = "gedcom"
    genie = "genie"
    genshi = "genshi"
    gentoo_ebuild = "gentoo-ebuild"
    gentoo_eclass = "gentoo-eclass"
    gerber_image = "gerber-image"
    gettext_catalog = "gettext-catalog"
    gherkin = "gherkin"
    git_attributes = "git-attributes"
    git_config = "git-config"
    glsl = "glsl"
    glyph = "glyph"
    glyph_bitmap_distribution_format = "glyph-bitmap-distribution-format"
    gn = "gn"
    gnuplot = "gnuplot"
    go = "go"
    golo = "golo"
    gosu = "gosu"
    grace = "grace"
    gradle = "gradle"
    grammatical_framework = "grammatical-framework"
    graph_modeling_language = "graph-modeling-language"
    graphql = "graphql"
    graphviz_dot = "graphviz-(dot)"
    groovy = "groovy"
    groovy_server_pages = "groovy-server-pages"
    hack = "hack"
    haml = "haml"
    handlebars = "handlebars"
    haproxy = "haproxy"
    harbour = "harbour"
    haskell = "haskell"
    haxe = "haxe"
    hcl = "hcl"
    hiveql = "hiveql"
    hlsl = "hlsl"
    holyc = "holyc"
    html_django = "html+django"
    html_ecr = "html+ecr"
    html_eex = "html+eex"
    html_erb = "html+erb"
    html_php = "html+php"
    html_razor = "html+razor"
    http = "http"
    hxml = "hxml"
    hy = "hy"
    hyphy = "hyphy"
    idl = "idl"
    idris = "idris"
    ignore_list = "ignore-list"
    igor_pro = "igor-pro"
    imagej_macro = "imagej-macro"
    inform_7 = "inform-7"
    ini = "ini"
    inno_setup = "inno-setup"
    io = "io"
    ioke = "ioke"
    irc_log = "irc-log"
    isabelle = "isabelle"
    isabelle_root = "isabelle-root"
    j = "j"
    jasmin = "jasmin"
    java = "java"
    java_properties = "java-properties"
    java_server_pages = "java-server-pages"
    javascript_erb = "javascript+erb"
    jflex = "jflex"
    jison = "jison"
    jison_lex = "jison-lex"
    jolie = "jolie"
    json = "json"
    json_with_comments = "json-with-comments"
    json5 = "json5"
    jsoniq = "jsoniq"
    jsonld = "jsonld"
    jsonnet = "jsonnet"
    julia = "julia"
    kaitai_struct = "kaitai-struct"
    kicad_layout = "kicad-layout"
    kicad_legacy_layout = "kicad-legacy-layout"
    kicad_schematic = "kicad-schematic"
    kit = "kit"
    kotlin = "kotlin"
    krl = "krl"
    labview = "labview"
    lasso = "lasso"
    latte = "latte"
    lean = "lean"
    less = "less"
    lex = "lex"
    lfe = "lfe"
    lilypond = "lilypond"
    limbo = "limbo"
    linker_script = "linker-script"
    linux_kernel_module = "linux-kernel-module"
    liquid = "liquid"
    literate_agda = "literate-agda"
    literate_coffeescript = "literate-coffeescript"
    literate_haskell = "literate-haskell"
    livescript = "livescript"
    llvm = "llvm"
    logos = "logos"
    logtalk = "logtalk"
    lolcode = "lolcode"
    lookml = "lookml"
    loomscript = "loomscript"
    lsl = "lsl"
    ltspice_symbol = "ltspice-symbol"
    lua = "lua"
    m = "m"
    m4 = "m4"
    m4sugar = "m4sugar"
    macaulay2 = "macaulay2"
    makefile = "makefile"
    mako = "mako"
    markdown = "markdown"
    marko = "marko"
    mask = "mask"
    mathematica = "mathematica"
    matlab = "matlab"
    maven_pom = "maven-pom"
    _max = "max"
    maxscript = "maxscript"
    mcfunction = "mcfunction"
    mediawiki = "mediawiki"
    mercury = "mercury"
    meson = "meson"
    metal = "metal"
    microsoft_developer_studio_project = "microsoft-developer-studio-project"
    minid = "minid"
    mirah = "mirah"
    mirc_script = "mirc-script"
    mlir = "mlir"
    modelica = "modelica"
    modula_2 = "modula-2"
    modula_3 = "modula-3"
    module_management_system = "module-management-system"
    monkey = "monkey"
    moocode = "moocode"
    moonscript = "moonscript"
    motorola_68k_assembly = "motorola-68k-assembly"
    mql4 = "mql4"
    mql5 = "mql5"
    mtml = "mtml"
    muf = "muf"
    mupad = "mupad"
    muse = "muse"
    mustache = "mustache"
    myghty = "myghty"
    nanorc = "nanorc"
    nasl = "nasl"
    ncl = "ncl"
    nearley = "nearley"
    nemerle = "nemerle"
    nesc = "nesc"
    netlinx = "netlinx"
    netlinx_erb = "netlinx+erb"
    netlogo = "netlogo"
    newlisp = "newlisp"
    nextflow = "nextflow"
    nginx = "nginx"
    nim = "nim"
    ninja = "ninja"
    nit = "nit"
    nix = "nix"
    nl = "nl"
    npm_config = "npm-config"
    nsis = "nsis"
    nu = "nu"
    numpy = "numpy"
    nunjucks = "nunjucks"
    nwscript = "nwscript"
    objdump = "objdump"
    object_data_instance_notation = "object-data-instance-notation"
    objective_c = "objective-c"
    objective_c__ = "objective-c++"
    objective_j = "objective-j"
    objectscript = "objectscript"
    ocaml = "ocaml"
    odin = "odin"
    omgrofl = "omgrofl"
    ooc = "ooc"
    opa = "opa"
    opal = "opal"
    open_policy_agent = "open-policy-agent"
    opencl = "opencl"
    openedge_abl = "openedge-abl"
    openqasm = "openqasm"
    openrc_runscript = "openrc-runscript"
    openscad = "openscad"
    openstep_property_list = "openstep-property-list"
    opentype_feature_file = "opentype-feature-file"
    org = "org"
    ox = "ox"
    oxygene = "oxygene"
    oz = "oz"
    p4 = "p4"
    pan = "pan"
    papyrus = "papyrus"
    parrot = "parrot"
    parrot_assembly = "parrot-assembly"
    parrot_internal_representation = "parrot-internal-representation"
    pascal = "pascal"
    pawn = "pawn"
    pep8 = "pep8"
    perl = "perl"
    php = "php"
    pic = "pic"
    pickle = "pickle"
    picolisp = "picolisp"
    piglatin = "piglatin"
    pike = "pike"
    plantuml = "plantuml"
    plpgsql = "plpgsql"
    plsql = "plsql"
    pod = "pod"
    pod_6 = "pod-6"
    pogoscript = "pogoscript"
    pony = "pony"
    postcss = "postcss"
    postscript = "postscript"
    pov_ray_sdl = "pov-ray-sdl"
    powerbuilder = "powerbuilder"
    powershell = "powershell"
    prisma = "prisma"
    processing = "processing"
    proguard = "proguard"
    prolog = "prolog"
    propeller_spin = "propeller-spin"
    protocol_buffer = "protocol-buffer"
    public_key = "public-key"
    pug = "pug"
    puppet = "puppet"
    pure_data = "pure-data"
    purebasic = "purebasic"
    purescript = "purescript"
    python_console = "python-console"
    python_traceback = "python-traceback"
    q = "q"
    q_23 = "q%23"
    qmake = "qmake"
    qml = "qml"
    qt_script = "qt-script"
    quake = "quake"
    r = "r"
    racket = "racket"
    ragel = "ragel"
    raku = "raku"
    raml = "raml"
    rascal = "rascal"
    raw_token_data = "raw-token-data"
    rdoc = "rdoc"
    readline_config = "readline-config"
    realbasic = "realbasic"
    reason = "reason"
    rebol = "rebol"
    red = "red"
    redcode = "redcode"
    regular_expression = "regular-expression"
    renpy = "ren'py"
    renderscript = "renderscript"
    rescript = "rescript"
    restructuredtext = "restructuredtext"
    rexx = "rexx"
    rich_text_format = "rich-text-format"
    ring = "ring"
    riot = "riot"
    rmarkdown = "rmarkdown"
    robotframework = "robotframework"
    roff = "roff"
    roff_manpage = "roff-manpage"
    rouge = "rouge"
    rpc = "rpc"
    rpm_spec = "rpm-spec"
    ruby = "ruby"
    runoff = "runoff"
    rust = "rust"
    sage = "sage"
    saltstack = "saltstack"
    sas = "sas"
    sass = "sass"
    scala = "scala"
    scaml = "scaml"
    scheme = "scheme"
    scilab = "scilab"
    scss = "scss"
    sed = "sed"
    self = "self"
    shaderlab = "shaderlab"
    shellsession = "shellsession"
    shen = "shen"
    sieve = "sieve"
    slash = "slash"
    _slice = "slice"
    slim = "slim"
    smali = "smali"
    smalltalk = "smalltalk"
    smarty = "smarty"
    smpl = "smpl"
    smt = "smt"
    solidity = "solidity"
    sourcepawn = "sourcepawn"
    sparql = "sparql"
    spline_font_database = "spline-font-database"
    sqf = "sqf"
    sql = "sql"
    sqlpl = "sqlpl"
    squirrel = "squirrel"
    srecode_template = "srecode-template"
    ssh_config = "ssh-config"
    stan = "stan"
    standard_ml = "standard-ml"
    starlark = "starlark"
    stata = "stata"
    ston = "ston"
    stylus = "stylus"
    subrip_text = "subrip-text"
    sugarss = "sugarss"
    supercollider = "supercollider"
    svelte = "svelte"
    svg = "svg"
    swift = "swift"
    swig = "swig"
    systemverilog = "systemverilog"
    tcl = "tcl"
    tcsh = "tcsh"
    tea = "tea"
    terra = "terra"
    tex = "tex"
    texinfo = "texinfo"
    text = "text"
    textile = "textile"
    thrift = "thrift"
    ti_program = "ti-program"
    tla = "tla"
    toml = "toml"
    tsql = "tsql"
    tsv = "tsv"
    tsx = "tsx"
    turing = "turing"
    turtle = "turtle"
    twig = "twig"
    txl = "txl"
    type_language = "type-language"
    unified_parallel_c = "unified-parallel-c"
    unity3d_asset = "unity3d-asset"
    unix_assembly = "unix-assembly"
    uno = "uno"
    unrealscript = "unrealscript"
    urweb = "urweb"
    v = "v"
    vala = "vala"
    vba = "vba"
    vbscript = "vbscript"
    vcl = "vcl"
    verilog = "verilog"
    vhdl = "vhdl"
    vim_help_file = "vim-help-file"
    vim_script = "vim-script"
    vim_snippet = "vim-snippet"
    visual_basic_net = "visual-basic-.net"
    volt = "volt"
    vue = "vue"
    wavefront_material = "wavefront-material"
    wavefront_object = "wavefront-object"
    wdl = "wdl"
    web_ontology_language = "web-ontology-language"
    webassembly = "webassembly"
    webidl = "webidl"
    webvtt = "webvtt"
    wget_config = "wget-config"
    windows_registry_entries = "windows-registry-entries"
    wisp = "wisp"
    wollok = "wollok"
    world_of_warcraft_addon_data = "world-of-warcraft-addon-data"
    x_bitmap = "x-bitmap"
    x_font_directory_index = "x-font-directory-index"
    x_pixmap = "x-pixmap"
    x10 = "x10"
    xbase = "xbase"
    xc = "xc"
    xcompose = "xcompose"
    xml = "xml"
    xml_property_list = "xml-property-list"
    xojo = "xojo"
    xpages = "xpages"
    xproc = "xproc"
    xquery = "xquery"
    xs = "xs"
    xslt = "xslt"
    xtend = "xtend"
    yacc = "yacc"
    yaml = "yaml"
    yang = "yang"
    yara = "yara"
    yasnippet = "yasnippet"
    zap = "zap"
    zeek = "zeek"
    zenscript = "zenscript"
    zephir = "zephir"
    zig = "zig"
    zil = "zil"
    zimpl = "zimpl"



import os

from string import Template

from ai_datahive.utils.text_helper import escape_html
from ai_datahive.utils.datetime_helper import today_as_start_and_enddate_str

from ai_datahive.collectors.models import Media
from ai_datahive.transformers.models import Content


class TopDailyImageTransformer:
    def __init__(self, template_file_name='top_daily_image_template.html', language='de'):
        from ai_datahive.dao.dao_factory import dao_factory
        self.creator = 'TopDailyImage'
        self.dao = dao_factory()

        self.language = language
        self.template_path = os.path.join(os.path.dirname(__file__), 'templates', template_file_name)

    def retrieve(self):
        start_date_str, end_date_str = today_as_start_and_enddate_str()

        filters = [["creator", "CivitAIDailyTopImage"], ['created_at', 'between', start_date_str, end_date_str]]
        media = self.dao.read(Media, filters, limit=1, order_by='likes')
        return media[0] if media else None

    def transform(self, topdailyimage: Media) -> Content:
        if topdailyimage:
            template_data = {
                'creator': topdailyimage.source,
                'media_url': topdailyimage.media_url,
                'likes': topdailyimage.likes,
                'hearts': topdailyimage.hearts,
                'prompt': topdailyimage.prompt,
                'model': topdailyimage.model,
                'author': topdailyimage.author,
                'created_at': topdailyimage.media_created_at,
            }
            str_content = self.create_content(template_data)

            content = Content(
                creator=self.creator,
                tags=topdailyimage.tags,
                title=f'Das Topbild des Tages von {topdailyimage.source}:',
                content=str_content,
                media_url=topdailyimage.media_url,
                media_type='image',
                media_created_at=topdailyimage.media_created_at,
                source=topdailyimage.source,
                language=self.language
            )

            return content

    def save(self, content: Content):
        result = self.dao.create(content)
        return result

    def create_content(self, template_data):
        template_data = {k: escape_html(v) for k, v in template_data.items()}

        with open(self.template_path, 'r', encoding='utf-8') as file:
            template_content = file.read()

        template = Template(template_content)
        return template.safe_substitute(template_data)

    def run(self):
        # get top image
        # check if top image was already top image
        # if yes try next top image until three tries
        # If all already top images write a message with the first one to say it is again the winner. in a row.
        media = self.retrieve()
        content = self.transform(media)
        self.save(content)


def main():
    from dotenv import load_dotenv
    load_dotenv()

    transformer = TopDailyImageTransformer()
    transformer.run()


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages


def read_requirements():
    with open('requirements.txt') as f:
        return [line.strip() for line in f.readlines() if not line.startswith('#')]


setup(
    name='ai_datahive',
    version='0.1.1',
    packages=find_packages(),
    description='ETL Package for crawling data, transform it with AI and store it in a datastorage.',
    author='Danny Gerst',
    author_email='d.gerst@bizrock.de',
    install_requires=read_requirements(),
)



import os

from string import Template


class PromptService:

    def __init__(self, prompt_path=None):
        if prompt_path:
            self.template_path = prompt_path
        else:
            self.template_path = os.path.join(os.path.dirname(__file__), 'prompts')

    def _create_prompt_file_path(self, prompt_file):
        return os.path.join(self.template_path, prompt_file)

    def _create_prompt_template(self, prompt_file=None, prompt=None):
        if not prompt_file and not prompt:
            ValueError('No prompt file or prompt string specified.')

        if not prompt:
            prompt = self.create_system_prompt_from_file(prompt_file)
        prompt_template = Template(prompt)

        return prompt_template

    def _create_prompt(self, prompt_template, prompt_data=None):
        if not prompt_data:
            prompt_data = {}

        return prompt_template.safe_substitute(prompt_data)

    def create_system_prompt_from_file(self, prompt_file):
        prompt_file_path = self._create_prompt_file_path(prompt_file)
        with open(prompt_file_path, 'r') as f:
            prompt = f.read()
        return prompt

    def create_translate_system_prompt(self, language=None, prompt_file='translate_prompt.txt', prompt=None):
        if not language:
            ValueError('No target language for the translation specified.')
        else:
            # read prompt from file relative directory prompts/translate_prompt.txt
            prompt_template = self._create_prompt_template(prompt_file, prompt)
            prompt_data = {'lang': language}
            prompt = self._create_prompt(prompt_template, prompt_data)

            return prompt

    def create_summarize_system_prompt(self, prompt_file='summarize_prompt.txt', prompt=None):
        prompt = self.create_system_prompt_from_file(prompt_file) if not prompt else prompt
        return prompt



from ai_datahive.services import PromptService, ai_service_factory

class AIBackedTranslationService:
    def __init__(self):
        self.llm_service = ai_service_factory.load_llm_service('TRANSLATION_LLM_SERVICE')
        self.ps = PromptService()

    def translate(self, text, target_language):
        system_msg = self.ps.create_translate_system_prompt(language=target_language)
        translation = self.llm_service.chat_response(system_msg, text)
        return translation



from ai_datahive.models.base_model import DataHiveBaseModel

from pydantic import field_validator


class ContentBaseModel(DataHiveBaseModel):
    creator: str
    source: str
    version: int = 0
    tags: str

    @field_validator('creator', mode='after')
    def check_strings(cls, v: str) -> str:
        if not isinstance(v, str):
            raise ValueError('Field must be a string')
        return v



from ai_datahive.dao.base_dao import BaseDAO


class DataService:
    def __init__(self, dao: BaseDAO):
        self.dao = dao

    # Beispiel: Hinzufügen einer neuen Entität
    def add_entity(self, entity):
        return self.dao.create(entity)

    # Füge hier weitere Geschäftslogiken hinzu

